/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  aria.h
//  aria
//
//  Created by Yohan Lasorsa on 02/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

/**
 \mainpage A2ML Reference Implementation API (ARIA)
 
 \section about About A2ML
 
 Advanced/Audio Mobile Markup Language (A2ML) is an XML format used for describing interactive audio scenes.
 It's a versatile and lightweight format based on a model/instance paradigm that can be used in many application domains: video games, guidance systems, user interfaces, interactive music, etc.

 More info on A2ML can be found on the Team WAM interactive audio project page: <a href="http://wam.inrialpes.fr/iaudio" target="_blank">http://wam.inrialpes.fr/iaudio</a>
 
 \section intro Introduction
 
 This API uses A2ML as its base input format. The primary concept of this API is to load interactive audio scenes described in A2ML using the ARIA API, build the corresponding audio objects hierachy, then communicate with the main program by the means of events.
 For most usages, simple or complex audio scenes can be completely managed using the event mechanism. When specific work must be done on audio objects, a DOM-like system allows the programmer to retrieve audio objects by IDs or by digging into the object hierarchy.
 There is 4 directly identifiable audio objects: cues, chunks, animations and sections. The properties of the playable audio objects (cues, chunks and sounds) are related to its synchronization to the audio scheduler. The audio parameters that affects the playback of these objects are modified throught Controls, for example the volume or pan.
 A Control object can be automatically animated using the Animation objects, but such behavior should be described in the A2ML documents.
 
 While all audio objects can be created programmatically, it is strongly discouraged to do so. The A2ML format should be used to describe the audio hierarchy and interactions, and audio objects of the API should be accessed only in specific cases where the complex parameter animation behavior of these objects can't be described straight into the A2ML document. 
 
 \section using_aria Using ARIA
 
 Most of the API usage is done through the Sound Manager object. It is a static class that controls all the behavior of the API, and also the starting point of any program using ARIA.
 
 Here is the basic scenario for ARIA usage:
 \code
 // -------------------------------------------------------------
 // App inits
 // -------------------------------------------------------------
 [AMSoundManager createSoundManager];
 NSError *myerror;
 [AMSoundManager loadA2ML:@"myfile.xml" error:&myerror];
 // ...

 // -------------------------------------------------------------
 // Main code
 // -------------------------------------------------------------
 AMScheduler *scheduler = [AMSoundManager getScheduler];
 [scheduler start];
 // do things...
 
 // -------------------------------------------------------------
 // App terminates
 // -------------------------------------------------------------
 [AMSoundManager releaseSoundManager];
 // ...
 \endcode
 
 This is the most basic usage of this API: it loads an A2ML document, and starts the scheduler.
 
 Now, if you need to send events to the scheduler, here is how it is done:
 \code
 NSNotificationCenter *notificationCenter = [AMSoundManager getNotificationCenter];
 [notificationCenter postNotificationName:@"example.myevent" object:self];
 \endcode
 
 Listening in your program for internally generated audio events, like when a cue ended, works in a similar way:
 \code
 NSNotificationCenter *notificationCenter = [AMSoundManager getNotificationCenter];
 [notificationCenter addObserver:self selector:@selector(mycallback) name:@"mycueid.end" object:nil];
 \endcode
 
 This way, each time the cue with the ID "mycueid" ends, the method "mycallback" will be called.
 Here is a list of some internal events you can listen to:
 - Start of a playable object: <em><id>.begin</em>
 - End of a playable object:  <em><id>.end</em>
 - Repetition of a playable object: <em><id>.repeat</em>
 
 For special cases where you might need advanced interaction with the audio objects described in the A2ML document, you can access them programmatically and access/modify their properties directly, for example:
 \code
 AMCue *mycue = [AMSoundManager getObjectFromID:@"mycueid"];
 mycue.loopCount = 2;	// change loop count value
 \endcode
 */
#import "a2ml.h"

// Types
#import "AMTime.h"
#import "AMValue.h"
#import "AMRandomValue.h"

// Protocols
#import "AMPlayable.h"
#import "AMControllable.h"
#import "AMControllableHierarchy.h"
#import "AMInstanciable.h"
#import "AMIdentifiable.h"
#import "AMParentIdentifiable.h"

// Parser
#import "AMParser.h"

// Controls
#import "AMControl.h"
#import "AMTempoControl.h"
#import "AMTriggerControl.h"
#import "AMSpawnControl.h"
#import "AMMix2DControl.h"
#import "AMMix3DControl.h"
#import "AMSoundControl.h"

// Instances
#import "AMInstance.h"
#import "AMSoundInstance.h"
#import "AMChunkInstance.h"
#import "AMCueInstance.h"
#import "AMAnimationInstance.h"

// Elements
#import "AMSound.h"
#import "AMChunk.h"
#import "AMCue.h"
#import "AMSection.h"
#import "AMAnimation.h"

// Scheduler
#import "AMScheduler.h"
#import "AMTimeAction.h"
#import "AMEventAction.h"

// General
#import "AMSoundManager.h"

