/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSoundManager.h
//  aria
//
//  Created by Yohan Lasorsa on 5/15/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMScheduler.h"
#import "AMSound.h"
#import "AMChunk.h"
#import "AMInstance.h"
#import "a2ml.h"
#import "fmod.hpp"
#import "fmod_errors.h"
#import <Foundation/Foundation.h>


/**
 The global sound manager.
 This class is meant to be used statically, and must not be instanciated.
 */
@interface AMSoundManager : NSObject 
{	
	// No instance variable as it's meant to be used as a static class.
}

/**
 Initialize the sound manager.
 */
+ (void)createSoundManager;

/**
 Destroy the sound manager and release all its resources.
 */
+ (void)releaseSoundManager;

/**
 Gets the A2ML scheduler.
 @return The internal A2ML scheduler.
 */
+ (AMScheduler *)getScheduler;

/**
 Load the given A2ML file.
 @param path The path of the A2ML file you want to load.
 @param error If loading failed at some point, gets the NSError describing the problem.
 */
+ (void)loadA2ML:(NSString *)_path error:(NSError **)_error;

/**
 Gets the notification center used for the internal event system.
 @return The internal NSNotificationCenter object used for events.
 */
+ (NSNotificationCenter *)getNotificationCenter;

/**
 Adds an unique string ID key for the corresponding object.
 @param id The string key for the provided object, must be unique.
 @param obj The object to match with a unique string.
 */
+ (void)addKeyID:(NSString *)_id forObject:(id)_obj;

/**
 Removes the string/object correspondence based on the specified string ID.
 @param id The string key of the correspondence to remove.
 */
+ (void)removeObjectForKeyID:(NSString *)_id;

/**
 Gets the object corresponding to the provided string ID.
 @param id The string key of the UID you want to retrieve.
 @return The object if the string ID match was found, or nil if no match was found.
 */
+ (id)getObjectFromID:(NSString *)_id;

/**
 Adds an unique string ID key for the corresponding UID.
 @param uid The UID key to match with a unique string.
 @param id The string for the provided UID, must be unique.
 */
+ (void)addKeyUID:(UID)_uid forID:(NSString *)_id;

/**
 Removes the UID/string correspondance based on the specified UID.
 @param uid The UID key of the correspondence to remove.
 */
+ (void)removeKeyUID:(UID)_uid;

/**
 Gets the ID string corresponding to the provided UID.
 @param uid The the UID of the ID string you want to retrieve.
 @return The ID string for the corresponding UID if it was found, or nil.
 */
+ (NSString *)getIDFromKeyUID:(UID)_uid;

/**
 Check if the provided string ID is already used.
 @param id The string ID to check.
 @return NO if the ID is already taken, YES otherwise.
 */
+ (BOOL)checkID:(NSString *)_id;

/**
 Adds an object instance to the release pool so it will be automatically
 released when it's not needed anymore.
 @param instance The object instance to add to the release pool.
 */
+ (void)addInstanceToReleasePool:(AMInstance *)_instance;

/**
 Removes an object instance from the release pool.
 @param instance The object instance to remove from the release pool.
 */
+ (void)removeInstanceFromReleasePool:(AMInstance *)_instance;

/**
 Gets the name of the controls supported.
 @return An NSSet object containing the names of the controls as NSString objects.
 */
+ (NSSet *)getControls;

/**
 Gets the control with the specified name.
 @param name The name of the control to get. Should be one of the constant control names defined in the AMControl class.
 @return The desired control, or nil if it does not exist.
 */
+ (AMControl *)getControlWithName:(NSString *)_name;

/**
 Gets the control with the specified name or creates it if it doesn't exist.
 @param name The name of the control to get or create. Should be one of the constant control names defined in the AMControl class.
 @return The desired control, or nil if it is not supported by the sound manager.
 */
+ (AMControl *)createControlWithName:(NSString *)_name;


/**
 Returns the internal FMOD system object.
 FOR INTERNAL USAGE ONLY.
 @return The internal FMOD system object.
 */
+ (FMOD::System *)getFMODSystem;

@end
