/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSoundManager.mm
//  aria
//
//  Created by Yohan Lasorsa on 5/15/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMSoundManager.h"
#import "AMParser.h"


// Class variables
static NSMutableDictionary *keyIDs;			/* Correspondance table for String ID => UID. */
static NSMutableDictionary *keyObj;			/* Correspondance table for String ID => object. */
static NSMutableDictionary *controls;		/* List of instanciated controls of the sound manager. */
static AMScheduler *scheduler;				/* The A2ML scheduler. */
static NSNotificationCenter *eventCenter;	/* The notification center used for events. */
static NSTimer *audioEngineUpdater;			/* Used to perferm regular actions for the audio engine. */
static NSTimer *instanceManagerUpdater;		/* Used to check regulary if instances placed in the pool needs to be deallocated. */
static NSMutableSet *instanceReleasePool;	/* Release pool for instances not currently playing */ 
static FMOD::System *fsystem;				/* The FMOD subsystem. */


@implementation AMSoundManager

+ (void)createSoundManager
{	
	AM_LOG_CORE(@"Init...");

	// init FMOD subsystem
	FMOD_RESULT result;
	
	result = FMOD::System_Create(&fsystem); 
	ERRCHECK(result);
	
	result = fsystem->setSoftwareChannels(128);		// Allow 128 software mixed voices to be audible at once.
	ERRCHECK(result);

	result = fsystem->init(512, FMOD_INIT_NORMAL /*| FMOD_INIT_ENABLE_PROFILE*/, NULL);
	ERRCHECK(result);
	
	// init vars
	keyIDs = [[NSMutableDictionary alloc] init];
	keyObj = [[NSMutableDictionary alloc] init];
	controls = [[NSMutableDictionary alloc] init];
	eventCenter = [[NSNotificationCenter alloc] init];
	instanceReleasePool = [[NSMutableSet alloc] init];
	
	// init scheduler
	scheduler = [[AMScheduler alloc] init];
	
	// init timers
	audioEngineUpdater = [NSTimer scheduledTimerWithTimeInterval:0.01 target:self selector:@selector(audioEngineUpdate:) userInfo:nil repeats:YES];
	instanceManagerUpdater = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(instanceManagerUpdate:) userInfo:nil repeats:YES];
	
	AM_LOG_CORE(@"Init OK.");
}

+ (void)releaseSoundManager
{
	// reset scheduler and stop all cues
	[scheduler resetAll];
	
	for (id obj in keyObj) {
		if ([obj isMemberOfClass:[AMCue class]])
			[(AMCue *)obj stop];
	}	
	
	// stop timers
	[instanceManagerUpdater invalidate];
	[audioEngineUpdater invalidate];
	
	// release vars
	[eventCenter release];
	[scheduler release];
	[keyIDs release];
	[keyObj release];
	[controls release];
	[instanceReleasePool release];
	
	fsystem->release();
	
	AM_LOG_CORE(@"Released, OK.");
}

+ (AMScheduler *)getScheduler
{
	return scheduler;
}

+ (void)loadA2ML:(NSString *)_path error:(NSError **)_error
{
	// reset scheduler and stop all cues
	[scheduler resetAll];

	for (id obj in keyObj) {
		if ([obj isMemberOfClass:[AMCue class]])
			[(AMCue *)obj stop];
	}
	
	// do some cleanup
	[keyIDs removeAllObjects];
	[keyObj removeAllObjects];
	[controls removeAllObjects];
	
	// load the new document
	AMParser *parser = [[AMParser alloc] init];
	[parser parseA2MLFileAtURL:_path parseError:_error];
	[parser release];
}

+ (NSNotificationCenter *)getNotificationCenter
{
	return eventCenter;
}

+ (void)audioEngineUpdate:(NSTimer *)timer 
{
	/** Main loop **/
	
	// update fmod subsystem
    FMOD_RESULT result = FMOD_OK;
    result = fsystem->update();
    ERRCHECK(result);
	
	// update scheduler
	if (![scheduler paused])
		[scheduler incrementTime]; 	
}

+ (void)instanceManagerUpdate:(NSTimer *)timer 
{
	NSMutableArray *toremove = [[NSMutableArray alloc] init];
	
	// check if some instance needs to be deallocated
	for (AMInstance *instance in instanceReleasePool) {
		
//		AM_LOG_CORE(@"Timeout to release cue instance [ID:%lld, PID:%lld]: %ds", [((AMCueInstance *)instance) getUID], [[((AMCueInstance *)instance) parent] getUID], ((AMCueInstance *)instance).releaseTimeout);
		
		// release cue instances when needed
		if ([instance isMemberOfClass:[AMCueInstance class]] && --(((AMCueInstance *)instance).releaseTimeout) <= 0) {
			
			AM_LOG_CORE(@"Killing instance ID:%lld", [((AMCueInstance *)instance) getUID]);
			[instance.parent destroyInstance:instance];
			[toremove addObject:instance];
		}
	}
	
	// remove destroyed instances from the pool
	for (AMInstance *instance in toremove)
		[instanceReleasePool removeObject:instance];
	
	[toremove release];
}

+ (void)addKeyID:(NSString *)_id forObject:(id)_obj
{
	[keyObj setObject:_obj forKey:_id];
}

+ (void)removeObjectForKeyID:(NSString *)_id;
{
	[keyObj removeObjectForKey:_id];
}

+ (id)getObjectFromID:(NSString *)_id
{
	return (NSObject *)[keyObj objectForKey:_id];
}

+ (void)addKeyUID:(UID)_uid forID:(NSString *)_id
{
	[keyIDs setObject:_id forKey:[NSNumber numberWithLongLong:_uid]];
}

+ (void)removeKeyUID:(UID)_uid
{
	[keyIDs removeObjectForKey:[NSNumber numberWithLongLong:_uid]];
}

+ (NSString *)getIDFromKeyUID:(UID)_uid;
{
	return [keyIDs objectForKey:[NSNumber numberWithLongLong:_uid]];
}

+ (BOOL)checkID:(NSString *)_id
{
	return (_id != nil && [[keyIDs allKeysForObject:_id] count] == 0);
}

+ (void)addInstanceToReleasePool:(AMInstance *)_instance
{
	// set release timeout
	if ([_instance class] == [AMCueInstance class])
		[(AMCueInstance *)_instance setReleaseTimeout:2];
	
	[instanceReleasePool addObject:_instance];
}

+ (void)removeInstanceFromReleasePool:(AMInstance *)_instance
{
	[instanceReleasePool removeObject:_instance];
}

+ (NSSet *)getControls
{
	return [NSSet setWithObjects:AM_TEMPO_CONTROL, AM_SPAWN_CONTROL, AM_MIX_2D_CONTROL, nil];
}

+ (AMControl *)getControlWithName:(NSString *)_name
{
	return [controls objectForKey:_name];
}

+ (AMControl *)createControlWithName:(NSString *)_name
{
	AMControl *control = [self getControlWithName:_name];
	
	// if the control has not been instanciated
	if (control == nil) {
		control = [AMControl controlWithName:_name controllable:self];
		
		// if the control is valid
		if (control != nil) {
			[controls setObject:control forKey:_name];
		}
	}
	
	return control;
}

+ (FMOD::System *)getFMODSystem
{
	return fsystem;
}

@end
