/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMAnimation.mm
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMAnimation.h"
#import "AMSound.h"


@implementation AMAnimation

@synthesize dur;
@synthesize quantize;
@synthesize from;
@synthesize to;
@synthesize by;
@synthesize fill;
@synthesize accumulation;
@synthesize repeatCount;
@synthesize repeatDur;
@synthesize values;
@synthesize eventValue;


- (id)initWithParent:(id<AMControllable>)_parent control:(NSString *)_control attribute:(NSString *)_attribute
{
	if (self = [super init]) {
		uid = GenerateUniqueID();
		parent = _parent;
		control = [_control copy];
		attribute = [_attribute copy];
		dur = nil;
		quantize = nil;
		from = nil;
		to = nil;
		by = nil;
		fill = AM_FIL_REMOVE;
		accumulation = AM_ACC_NONE;
		repeatDur = nil;
		repeatCount = 1.0;
		values = nil;
		eventValue = nil;
		
		instances = [[NSMutableArray alloc] init];
	}
	return self;
}

- (UID)getUID
{
	return uid;
}

- (AMControl *)getClosestControlWithName:(NSString *)_name
{
	return [(id<AMControllableHierarchy>)parent getClosestControlWithName:_name];
}

/*
 Internal function.
 Gets the cue instance corresponding to the given parent UID.
 */
- (AMAnimationInstance *)getInstanceFromParentUid:(UID)_uid
{
	AMAnimationInstance *instance = nil;
	int i, count = [instances count];
	
	// search for the desired instance
	for (i = 0; i < count; i++) {
		instance = [instances objectAtIndex:i];
		if ([instance getParentUID] == _uid)
			break;
	}
	
	// check if the id was found
	if (i == count) {
		instance = nil;
	}
	
	return instance;
}

- (void)animateWithValues:(id)_value 
{	
	AMControl *ctl = [parent createControlWithName:control];
	[ctl setValue:_value forAttribute:[ctl attributeCode:attribute]];
	
	// check if new settings needs to be applied
	if ((![(NSObject *)parent isKindOfClass:[AMInstance class]]) && ([control isEqual:AM_MIX_2D_CONTROL] || [control isEqual:AM_MIX_3D_CONTROL] || [control isEqual:AM_SOUND_CONTROL])) {
		
		// TODO: fix better
		if ([(NSObject *)parent isMemberOfClass:[AMSound class]]) {
			
			
			AMSound *snd = (AMSound *)parent;
			NSLog(@"src %@", [snd src]);
			for (AMInstance *instance in [snd getInstances]) {				
				if ([ctl respondsToSelector:@selector(applySettingsToInstance:)])
					[ctl performSelector:@selector(applySettingsToInstance:) withObject:instance];
			}
			
		}
		
	}
}

- (id)start
{
	AMAnimationInstance *instance = (AMAnimationInstance *)[self createInstance];
	
	if (instance != nil)
		[instance start];
	
	return instance;
}

- (id)startFromParentUid:(UID)_uid listener:(id)_listener
{
	AMAnimationInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance == nil)
		instance = (AMAnimationInstance *)[self createInstanceWithParentUid:_uid];
	[instance start];
	
	return instance;
}

- (id)stop
{
	// stop all instances
	AMAnimationInstance *instance = nil;
	
	for (instance in instances) {
		[instance stop];
	}
	
	return instance;
}

- (id)stopFromParentUid:(UID)_uid
{
	AMAnimationInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		[instance stop];
	
	return instance;
}

- (BOOL)isPlaying
{
	BOOL playing = false;
	
	for (AMAnimationInstance *instance in instances) {
		if ([instance isPlaying]) {
			playing = true;
			break;
		}
	}
	
	return playing;
}

- (BOOL)isPlayingFromParentUid:(UID)_uid
{
	AMAnimationInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		return [instance isPlaying];
	
	return NO;
}

- (double)getDuration
{
	// duration is undefined here, only valid in animation instance
	return -1;
}

- (double)getDurationFromParentUid:(UID)_uid
{
	AMAnimationInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		return [instance getDuration];
	
	return NO;
}

- (id)createInstance
{
	AMAnimationInstance *instance = nil;
	
	//TODO: Check if it is possible to create the instance
	
	instance = [[AMAnimationInstance alloc] initWithParent:self parentUid:GenerateUniqueID()];
	[instances addObject:instance];
	[instance autorelease];
	
	return instance;
}

- (id)createInstanceWithParentUid:(UID)_uid
{
	AMAnimationInstance *instance = nil;
	
	//TODO: Check if it is possible to create the instance
	
	instance = [[AMAnimationInstance alloc] initWithParent:self parentUid:_uid];
	[instances addObject:instance];
	[instance autorelease];
	
	return instance;
}

- (void)destroyInstance:(id)_instance
{
	[instances removeObject:_instance];
}

- (void)destroyInstanceFromParentUid:(UID)_uid
{
	AMAnimationInstance *instance = [self getInstanceFromParentUid:_uid];
	if (instance) {
		[instances removeObject:instance];
	}
}

- (id<AMControllable>)getControlParent 
{
	return parent;
}
- (NSString *)getControlName
{
	return control;
}

- (NSString *)getAttributeName
{
	return attribute;
}

-(void)dealloc 
{
	// release vars	
	self.dur = nil;
	self.quantize = nil;
	self.from = nil;
	self.to = nil;
	self.by = nil;
	self.repeatDur = nil;
	self.values = nil;
	self.eventValue = nil;
	
	while ([instances count] > 0) {
		[self destroyInstance:[instances lastObject]];
	}
	[control release];
	[attribute release];
	[instances release];
	
	[super dealloc];
}

@end
