/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMChunk.h
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "a2ml.h"
#import "AMTime.h"
#import "AMSound.h"
#import "AMControllable.h"
#import "AMControllableHierarchy.h"
#import "AMPlayable.h"
#import "AMInstanciable.h"
#import "AMIdentifiable.h"
#import "AMChunkInstance.h"
#import <Foundation/Foundation.h>


/**
	An audio fragment embedding one ore more tracks that can be played in defined order or randomly.
 */
@interface AMChunk : NSObject <AMControllable, AMControllableHierarchy, AMPlayable, AMInstanciable, AMIdentifiable> {
	// Chunk attributes
	UID uid;
	AMTime *dur;
	AMPickType pick;
	AMFadeInType fadeInType;
	AMTime *fadeInDur;
	AMFadeOutType fadeOutType;
	AMTime *fadeOutDur;
	int activeSound;
	int nextSound;
	
	// Internal variables
	id parent;						/* Parent object of the chunk. */
	NSMutableArray *instances;		/* List of current instances of this chunk. */
	NSMutableDictionary *controls;	/* List of instanciated controls for this chunk. */
	NSMutableArray *sounds;			/* The sounds of this chunk. */
}

/**
 The duration of the chunk.
 */
@property (nonatomic, copy) AMTime *dur;

/**
 Determines how the sounds are picked.
 */
@property (nonatomic) AMPickType pick;

/**
 The fade in type for the chunk.
 If the previous chunk has set AM_FOT_CROSSFADE as it fade out type, the fade in properties for
 this chunk will be ignored.
 */
@property (nonatomic) AMFadeInType fadeInType;

/**
 The fade in duration.
 */
@property (nonatomic, copy) AMTime *fadeInDur;

/**
 The fade out type for the transition with the next chunk.
 */
@property (nonatomic) AMFadeOutType fadeOutType;

/**
 The duration of the fade out transition.
 */
@property (nonatomic, copy) AMTime *fadeOutDur;

/**
 The index of the current active sound.
 */
@property (nonatomic) int activeSound;

/**
 The index of the nexy active sound.
 */
@property (nonatomic) int nextSound;

/**
 Creates a new chunk object.
 @param parent The parent object of the chunk.
 @return The created chunk object.
 */
- (id)initWithParent:(id)_parent;

/**
 Adds a sound to the chunk.
 @param sound The AMSound object to add.
 */
- (void)addSound:(AMSound *)_sound;

/**
 Removes the sound at the given index from the chunk.
 The function is ignored if the index is invalid.
 @param idx The index of the sound object to remove.
 */
- (void)removeSoundAtIndex:(int)_idx;

/**
 Sets the sound as active.
 Has no effects on currently playing instances of the chunk.
 @param sound The sound to set as active.
 */
- (void)setActive:(AMSound *)_sound;


/**
 Returns the internal array of sounds.
 FOR INTERNAL USAGE ONLY.
 @return The internal array of sounds.
 */
- (NSArray *)getSounds;

@end
