/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMChunk.mm
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMChunk.h"


@implementation AMChunk

@synthesize dur;
@synthesize pick;
@synthesize fadeInType;
@synthesize fadeInDur;
@synthesize fadeOutType;
@synthesize fadeOutDur;
@synthesize activeSound;
@synthesize nextSound;

- (id)initWithParent:(id)_parent
{
	if (self = [super init]) {
		parent = _parent;
		uid = GenerateUniqueID();
		dur = nil;
		pick = AM_PT_ORDERED;
		fadeInType = AM_FIT_NONE;
		fadeInDur = nil;
		fadeOutType = AM_FOT_NONE;
		fadeOutDur = nil;
		activeSound = 0;
		nextSound = -1;
		
		instances = [[NSMutableArray alloc] init];
		sounds = [[NSMutableArray alloc] init];
		controls = nil;
	}
	return self;
}

- (UID)getUID
{
	return uid;
}

/*
 Internal function.
 Gets the chunk instance corresponding to the given parent UID.
 */
- (AMChunkInstance *)getInstanceFromParentUid:(UID)_uid
{
	AMChunkInstance *instance = nil;
	int i, count = [instances count];
	
	// search for the desired instance
	for (i = 0; i < count; i++) {
		instance = [instances objectAtIndex:i];
		if ([instance getParentUID] == _uid)
			break;
	}
	
	// check if the id was found
	if (i == count) {
		instance = nil;
	}
	
	return instance;
}

- (void)addSound:(AMSound *)_sound
{
	[sounds addObject:_sound];
	
	//TODO: calc nextChunk, updateDUr? NO!!! instance but check for instances
	
	
}


- (void)removeSoundAtIndex:(int)_idx
{
	[sounds removeObjectAtIndex:_idx];
	
	//TODO: calc nextChunk, updateDUr?  NO!!! instance but check for instances

}

- (void)setActive:(AMSound *)_sound
{
	for (int i=0; i < [sounds count]; i++) {
		if (_sound == (AMSound *)[sounds objectAtIndex:i]) {
			activeSound = i;
			break;
		}
	}
}

- (id)start
{
	AMChunkInstance *instance = (AMChunkInstance *)[self createInstance];
	
	if (instance != nil)
		[instance start];
	
	return instance;
}

- (id)startFromParentUid:(UID)_uid listener:(id)_listener
{
	AMChunkInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil) {
		[instance setListener:_listener];
		[instance start];
	} else {
		instance = (AMChunkInstance *)[self createInstanceWithParentUid:_uid];
		[instance setListener:_listener];
		[instance start];
	}
	
	return instance;
}

- (id)stop
{
	// stop all instances
	AMCueInstance *instance = nil;
	
	for (instance in instances) {
		[instance stop];
	}
	
	return instance;
}

- (id)stopFromParentUid:(UID)_uid
{
	AMChunkInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		[instance stop];
	
	return instance;
}

- (BOOL)isPlaying
{
	BOOL playing = false;
	
	for (AMChunkInstance *instance in instances) {
		if ([instance isPlaying]) {
			playing = true;
			break;
		}
	}
	
	return playing;
}

- (BOOL)isPlayingFromParentUid:(UID)_uid
{
	AMChunkInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		return [instance isPlaying];
	
	return NO;
}

- (double)getDuration
{
	// you can't get the duration by there since its dependant of the instance
	return -1;
}

- (double)getDurationFromParentUid:(UID)_uid
{
	AMChunkInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		return [instance getDuration];
	
	return NO;
}

- (NSSet *)getControls
{
	return [NSSet setWithObjects:AM_TEMPO_CONTROL, nil];
}

- (AMControl *)getControlWithName:(NSString *)_name
{
	return [controls objectForKey:_name];
}

- (AMControl *)createControlWithName:(NSString *)_name
{
	AMControl *control = [self getControlWithName:_name];
	
	// if the control has not been instanciated
	if (control == nil) {
		control = [AMControl controlWithName:_name controllable:self];
		
		// if the control is valid
		if (control != nil) {
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];						
			[controls setObject:control forKey:_name];
		}
	}
	
	return control;
}

- (AMControl *)getClosestControlWithName:(NSString *)_name
{
	AMControl *control = [controls objectForKey:_name];
	
	if (control == nil) {
		control = [parent getClosestControlWithName:_name];
	} else {
		// keeps the object valid if the sound instance gets killed.
		[[control retain] autorelease];
	}
	
	return control;
}

- (id)createInstance
{
	AMChunkInstance *instance = nil;
	
	//TODO: Check if it is possible to create the instance
	
	instance = [[AMChunkInstance alloc] initWithParent:self parentUid:UNDEFINED_UID];
	[instances addObject:instance];
	[instance autorelease];
	
	return instance;
}

- (id)createInstanceWithParentUid:(UID)_uid
{
	AMChunkInstance *instance = nil;
	
	//TODO: Check if it is possible to create the instance
	
	instance = [[AMChunkInstance alloc] initWithParent:self parentUid:_uid];
	[instances addObject:instance];
	[instance autorelease];
	
	return instance;
}

- (void)destroyInstance:(id)_instance
{
	[instances removeObject:_instance];
}

- (void)destroyInstanceFromParentUid:(UID)_uid
{
	AMChunkInstance *instance = [self getInstanceFromParentUid:_uid];
	if (instance) {
		[instances removeObject:instance];
	}
}

- (NSArray *)getSounds
{
	return sounds;
}

-(void)dealloc 
{
	// release vars
	
	// TODO CHECK: release each instance is ok?
	while ([instances count] > 0) {
		[self destroyInstance:[instances lastObject]];
	}
	[instances release];
	[controls release];
	[sounds release];
	
	[super dealloc];
}

@end
