/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMCue.h
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "a2ml.h"
#import "AMTime.h"
#import "AMChunk.h"
#import "AMControllable.h"
#import "AMControllableHierarchy.h"
#import "AMPlayable.h"
#import "AMInstanciable.h"
#import "AMIdentifiable.h"
#import "AMCueInstance.h"
#import <Foundation/Foundation.h>


// to avoid import loop errors
@class AMChunk;
@class AMCueInstance;

/**
 A managed sound source controlled by time and/or events.
 It is a container for one or more audio chunks.
 */
@interface AMCue : NSObject <AMControllable, AMControllableHierarchy, AMPlayable, AMInstanciable, AMIdentifiable> {
	// Cue attributes
	UID uid;
	AMTime *dur;
	AMTime *quantize;
	int loopCount;
	int priority;
	AMRetriggerType restartOnRetrigger;
	BOOL stream;
	
	// Internal variables
	NSMutableArray *instances;		/* List of current instances of this cue. */
	NSMutableDictionary *controls;	/* List of instanciated controls for this cue. */
	NSMutableArray *chunks;			/* The chunks of this cue. */
	NSMutableArray *channelGroups;	/* The section (= FMOD channel groups) the cue belongs to. */
}

/**
 The duration of the cue.
 */
@property (nonatomic, copy) AMTime *dur;

/**
 The quantization value of the cue.
 */
@property (nonatomic, copy) AMTime *quantize;

/**
 Number of times the cue will loop and play the content of its audio source.
 If set to -1, the cue will repeat indefinitely.
 */
@property (nonatomic) int loopCount;

/**
 The priority of the cue, between 0 and 100.
 */
@property (nonatomic) int priority;

/**
 Defines the behavior of the cue when a start request is trigerred while it is already playing.
 */
@property (nonatomic) AMRetriggerType restartOnRetrigger;

/**
 Defines how the audio elements of the cue will use the memory.
 Note that streaming activation will also limit the cue to 1 instance at a time.
 */
@property (nonatomic) BOOL stream;


/**
 Creates a new cue object.
 @return The created cue object.
 */
- (id)init;

/**
 Adds a chunk to the cue.
 @param chunk The AMChunk object to add.
 */
- (void)addChunk:(AMChunk *)_chunk;

/**
 Removes the chunk at the given index from the cue.
 The function is ignored if the index is invalid.
 @param idx The index of the chunk object to remove.
 */
- (void)removeChunkAtIndex:(int)_idx;


/**
 Returns the internal array of chunks.
 FOR INTERNAL USAGE ONLY.
 @return The internal array of chunks.
 */
- (NSArray *)getChunks;

/**
 Returns the internal array of instances.
 FOR INTERNAL USAGE ONLY.
 @return The internal array of instances.
 */
- (NSArray *)getInstances;

/**
 Returns the internal array pointer of FMOD channel groups.
 FOR INTERNAL USAGE ONLY.
 @return the internal array pointer of FMOD channel groups.
 */
- (NSMutableArray **)getFMODChannelGroups;

/**
 Returns the internal cue instance corresponding to the given parent UID.
 FOR INTERNAL USAGE ONLY.
 @return The internal cue instance corresponding to the given parent UID.
 */
- (AMCueInstance *)getCueInstanceWithParentUID:(UID)_parentUID;

@end
