/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMCue.mm
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMCue.h"


@implementation AMCue

@synthesize dur;
@synthesize quantize;
@synthesize loopCount;
@synthesize priority;
@synthesize restartOnRetrigger;
@synthesize stream;

- (id)init
{
	if (self = [super init]) {
		uid = GenerateUniqueID();
		dur = nil;
		quantize = nil;
		loopCount = 1;
		priority = 1;
		restartOnRetrigger = AM_RT_AUTO;
		stream = false;
		
		instances = [[NSMutableArray alloc] init];
		chunks = [[NSMutableArray alloc] init];
		controls = nil;
		channelGroups = nil;
	}
	return self;
}

- (UID)getUID
{
	return uid;
}

/*
 Internal function.
 Gets the cue instance corresponding to the given parent UID.
 */
- (AMCueInstance *)getInstanceFromParentUid:(UID)_uid
{
	AMCueInstance *instance = nil;
	int i, count = [instances count];
	
	// search for the desired instance
	for (i = 0; i < count; i++) {
		instance = [instances objectAtIndex:i];
		if ([instance getParentUID] == _uid)
			break;
	}
	
	// check if the id was found
	if (i == count) {
		instance = nil;
	}
	
	return instance;
}

- (void)addChunk:(AMChunk *)_chunk
{
	[chunks addObject:_chunk];
	
	for (AMCueInstance *instance in instances) {
		[instance updateNextChunk];
	}
}

- (void)removeChunkAtIndex:(int)_idx
{
	[chunks removeObjectAtIndex:_idx];
}

- (id)start
{
	AMCueInstance *instance = (AMCueInstance *)[self createInstance];
	
	if (instance != nil)
		[instance start];
	
	return instance;
}

- (id)startFromParentUid:(UID)_uid listener:(id)_listener
{
	AMCueInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil) {
		[instance setListener:_listener];
		[instance start];
	} else {
		instance = (AMCueInstance *)[self createInstanceWithParentUid:_uid];
		[instance setListener:_listener];
		[instance start];
	}
	
	return instance;
}

- (id)stop
{
	// stop all instances
	AMCueInstance *instance = nil;
	
	for (instance in instances) {
		[instance stop];
	}
	
	return instance;
}

- (id)stopFromParentUid:(UID)_uid
{
	AMCueInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		[instance stop];
	
	return instance;
}

- (BOOL)isPlaying
{
	BOOL playing = false;
	
	for (AMCueInstance *instance in instances) {
		if ([instance isPlaying]) {
			playing = true;
			break;
		}
	}
	
	return playing;
}

- (BOOL)isPlayingFromParentUid:(UID)_uid
{
	AMCueInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		return [instance isPlaying];
	
	return NO;
}

- (double)getDuration
{
	// you can't get the duration by there since its dependant of the instance
	return -1;
}

- (double)getDurationFromParentUid:(UID)_uid
{
	AMCueInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil)
		return [instance getDuration];
	
	return NO;
}

- (NSSet *)getControls
{
	return [NSSet setWithObjects:AM_TEMPO_CONTROL, AM_TRIGGER_CONTROL, AM_SPAWN_CONTROL, AM_MIX_3D_CONTROL, nil];
}

- (AMControl *)getControlWithName:(NSString *)_name
{
	return [controls objectForKey:_name];
}

- (AMControl *)createControlWithName:(NSString *)_name
{
	AMControl *control = [self getControlWithName:_name];
	
	// if the control has not been instanciated
	if (control == nil) {
		control = [AMControl controlWithName:_name controllable:self];
		
		// if the control is valid
		if (control != nil) {
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];						
			[controls setObject:control forKey:_name];
		}
	}
	
	return control;
}

- (AMControl *)getClosestControlWithName:(NSString *)_name
{
	AMControl *control = [controls objectForKey:_name];
	
	if (control == nil) {
		control = [AMSoundManager createControlWithName:_name];
	} else {
		// keeps the object valid if the cue gets killed.
		[[control retain] autorelease];
	}
	
	return control;
}

- (id)createInstance
{
	AMCueInstance *instance = nil;
	
	//TODO: Check if it is possible to create the instance
	
	instance = [[AMCueInstance alloc] initWithParent:self parentUid:GenerateUniqueID()];
	[instances addObject:instance];
	[instance autorelease];
	
	return instance;
}

- (id)createInstanceWithParentUid:(UID)_uid
{
	AMCueInstance *instance = nil;
	
	//TODO: Check if it is possible to create the instance
	
	instance = [[AMCueInstance alloc] initWithParent:self parentUid:_uid];
	[instances addObject:instance];
	[instance autorelease];
	
	return instance;
}

- (void)destroyInstance:(id)_instance
{
	// destroy all related chunk instances
	for (AMChunk *chunk in chunks)
		[chunk destroyInstanceFromParentUid:[_instance getParentUID]];
	
	[instances removeObject:_instance];
}

- (void)destroyInstanceFromParentUid:(UID)_uid
{
	AMCueInstance *instance = [self getInstanceFromParentUid:_uid];
	if (instance) {
		[instances removeObject:instance];
	}
}

- (NSArray *)getChunks
{
	return chunks;
}

- (NSArray *)getInstances
{
	return instances;
}

- (NSMutableArray **)getFMODChannelGroups
{
	return &channelGroups;
}

- (AMCueInstance *)getCueInstanceWithParentUID:(UID)_parentUID
{
	AMCueInstance *cueInstance = nil;
	
	if (_parentUID != UNDEFINED_UID) {
		for (AMCueInstance *inst in instances) {
			if ([inst getParentUID] == _parentUID) {
				cueInstance = inst;
				break;
			}
		}
	}
	
	return cueInstance;
}

-(void)dealloc 
{
	// release vars
	
	// TODO CHECK: release each instance is ok?
	while ([instances count] > 0) {
		[self destroyInstance:[instances lastObject]];
	}
	[instances release];
	[controls release];
	[channelGroups release];
	[chunks release];
	
	AM_LOG_CORE(@"[ID:%lld] Released.", uid);

	[super dealloc];
}

@end
