/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSound.h
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMControllable.h"
#import "AMControllableHierarchy.h"
#import "AMPlayable.h"
#import "AMInstanciable.h"
#import "AMIdentifiable.h"
#import "AMSoundManager.h"
#import "AMSoundInstance.h"
#import "AMCue.h"
#import "AMChunk.h"
#import "AMControl.h"
#import "fmod.hpp"
#import "fmod_errors.h"
#import <Foundation/Foundation.h>


// to avoid import loop errors
@class AMCue;
@class AMChunk;

/**
	The most basic audio element object of the hierarchy.
	It can be viewed as a container for a defined atomic sound.
 */
@interface AMSound : NSObject <AMControllable, AMControllableHierarchy, AMPlayable, AMInstanciable, AMIdentifiable> {
	// Sound attributes
	int loopCount;
	int pickPriority;
	NSString *src;
	
	// Internal variables
	UID uid;						/* Unique object ID. */
	id parent;						/* Parent object of the sound. */
	AMCue *parentCue;				/* The parent cue the sound belongs to. */
	int remainingLoops;				/* Number of remaining loops to play. */
	NSMutableArray *instances;		/* List of current instances of this sound. */
	NSMutableDictionary *controls;	/* List of instanciated controls for this sound. */
	FMOD::Sound *snd;				/* The current sound of this sound. */
}

/**
 Number of times the sound will loop and play the content of its audio source.
 If set to -1, the sound will repeat indefinitely.
 */
@property (nonatomic) int loopCount;											

/**
 The probability of this sound to be picked betwwen all sounds of the chunk if the chunk pick mode is set to random.
 Probability is defined as relative to other sounds, and must be positive.
 */
@property (nonatomic) int pickPriority;

/**
 The sound source of the sound.
 */
@property (nonatomic, readonly) NSString *src;

/**
 Creates a new sound object.
 @param parent The parent object of the sound.
 @param parentCue The parent cue object of this sound.
 @param src The sound source for the sound.
 @return The created sound object.
 */
- (id)initWithParent:(id)_parent parentCue:(AMCue *)_parentCue src:(NSString *)_src;

/**
 Sets the sound as the active one in its parent chunk.
 Has no effects on currently playing instances of the chunk.
 */
- (void)setActive;


/**
 Returns the internal FMOD sound object.
 FOR INTERNAL USAGE ONLY.
 @return The internal FMOD sound object.
 */
- (FMOD::Sound *)getFMODSound;
 

/**
 Returns the parent cue the sound belongs to.
 FOR INTERNAL USE ONLY.
 @return The parent cue the sound belongs to.
 */
- (AMCue *)getParentCue;

/**
 Returns the instances of the sound.
 FOR INTERNAL USE ONLY.
 @return The instances of the sound.
 */
- (NSArray *)getInstances;


@end
