/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSound.mm
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMSound.h"


@implementation AMSound

@synthesize loopCount;											
@synthesize pickPriority;
@synthesize src;

- (id)initWithParent:(id)_parent parentCue:(AMCue *)_parentCue src:(NSString *)_src
{	
	if (self = [super init]) {
		parent = _parent;
		parentCue = _parentCue;
		src = [_src copy];
		loopCount = 1;
		pickPriority = 1;
		controls = nil;
		remainingLoops = loopCount;
		instances = nil;
		uid = GenerateUniqueID();
		instances = [[NSMutableArray alloc] init];
		
		// init FMOD sound
		FMOD_RESULT result;
		
		if (_parentCue.stream) 
			result = [AMSoundManager getFMODSystem]->createSound([src cStringUsingEncoding:NSASCIIStringEncoding], FMOD_CREATESTREAM | FMOD_SOFTWARE, NULL, &snd);		// FMOD_DEFAULT uses the defaults.  These are the same as FMOD_LOOP_OFF | FMOD_2D | FMOD_HARDWARE.
		else
			result = [AMSoundManager getFMODSystem]->createSound([src cStringUsingEncoding:NSASCIIStringEncoding], FMOD_CREATECOMPRESSEDSAMPLE | FMOD_SOFTWARE, NULL, &snd);		
		
		ERRCHECK(result);
	}
	return self;
}

- (UID)getUID
{
	return uid;
}

- (void)setActive
{	
	[(AMChunk *)parent setActive:self];
	
	// TODO: broadcast setActive event
}

/*
 Internal function.
 Gets the sound instance corresponding to the given parent UID.
 */
- (AMSoundInstance *)getInstanceFromParentUid:(UID)_uid
{
	AMSoundInstance *instance = nil;
	int i, count = [instances count];
	
	// search for the desired instance
	for (i = 0; i < count; i++) {
		instance = [instances objectAtIndex:i];
		if ([instance getParentUID] == _uid)
			break;
	}
	
	// check if the id was found
	if (i == count) {
		instance = nil;
	}
	
	return instance;
}

- (id)start 
{
	AMSoundInstance *instance = [self createInstance];
	[instance start];
	return instance;
}

- (id)startFromParentUid:(UID)_uid listener:(id)_listener
{
	AMSoundInstance *instance = [self createInstanceWithParentUid:_uid];
	[instance setListener:_listener];
	[instance start];
	return instance;
}

- (id)stop
{
	// stop all instances
	AMCueInstance *instance = nil;
	
	for (instance in instances) {
		[instance stop];
	}
	
	return instance;
}

- (id)stopFromParentUid:(UID)_uid
{
	AMSoundInstance *instance = [self getInstanceFromParentUid:_uid];

	if (instance != nil) {
		[instance kill];
	}
	
	return instance;
}

- (BOOL)isPlaying
{
	BOOL result = NO;
	
	// if one instance is present then it's playing
	result = [instances count] > 0;
	
	return result;
}

- (BOOL)isPlayingFromParentUid:(UID)_uid
{
	AMSoundInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil) {
		return YES;
	}
	
	return NO;
}

- (double)getDuration
{
	unsigned int len;
	double dur = -1;
	
	FMOD_RESULT result;
	result = snd->getLength(&len, FMOD_TIMEUNIT_MS);
	ERRCHECK(result);
	
	if (len != 0xFFFFFFFF) {
		dur = (double)len / 1000.0;
	}
	
	return dur;
}

- (double)getDurationFromParentUid:(UID)_uid
{
	AMSoundInstance *instance = [self getInstanceFromParentUid:_uid];
	
	if (instance != nil) {
		return [instance getDuration];
	}
	
	return -1;
}

- (NSSet *)getControls
{
	return [NSSet setWithObjects:AM_TEMPO_CONTROL, AM_MIX_2D_CONTROL, AM_SOUND_CONTROL, nil];
}

- (AMControl *)getControlWithName:(NSString *)_name
{
	return [controls objectForKey:_name];
}

- (AMControl *)createControlWithName:(NSString *)_name
{
	AMControl *control = [self getControlWithName:_name];
	
	// if the control has not been instanciated
	if (control == nil) {
		control = [AMControl controlWithName:_name controllable:self];
		
		// if the control is valid
		if (control != nil) {
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];						
			[controls setObject:control forKey:_name];
		}
	}
	
	return control;
}

- (AMControl *)getClosestControlWithName:(NSString *)_name
{
	AMControl *control = [controls objectForKey:_name];
	
	if (control == nil) {
		control = [parent getClosestControlWithName:_name];
	} else {
		// keeps the object valid if the sound instance gets killed.
		[[control retain] autorelease];
	}
	
	return control;
}

- (id)createInstance
{
	AMSoundInstance *instance = nil;

	//TODO: Check if it is possible to create the instance
	
	instance = [[AMSoundInstance alloc] initWithParent:self parentUid:UNDEFINED_UID];
	[instances addObject:instance];
	
	return instance;
}

- (id)createInstanceWithParentUid:(UID)_uid
{
	AMSoundInstance *instance = nil;
	
	//TODO: Check if it is possible to create the instance
	
	instance = [[AMSoundInstance alloc] initWithParent:self parentUid:_uid];
	[instances addObject:instance];
	
	return instance;
}

- (void)destroyInstance:(id)_instance
{
	[instances removeObject:_instance];
	[_instance release];
}

- (void)destroyInstanceFromParentUid:(UID)_uid
{
	AMSoundInstance *instance = [self getInstanceFromParentUid:_uid];
	if (instance) {
		[instances removeObject:instance];
		[instance release];
	}
}

- (FMOD::Sound *)getFMODSound
{
	return snd;
}

- (AMCue *)getParentCue
{
	return parentCue;
}

- (NSArray *)getInstances
{
	return instances;
}

-(void)dealloc 
{
	// release vars
	snd->release();
	[src release];
	
	// TODO CHECK: release each instance is ok?
	while ([instances count] > 0) {
		[self destroyInstance:[instances lastObject]];
	}
	[instances release];
	[controls release];
	
	[super dealloc];
}

@end
