/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMAnimationInstance.mm
//  aria
//
//  Created by Yohan on 11/08/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMSoundManager.h"
#import "AMTempoControl.h"
#import "AMAnimationInstance.h"


@implementation AMAnimationInstance

- (id)initWithParent:(id)_parent parentUid:(UID)_parentUid
{
	if (self = [super initWithParent:_parent parentUid:_parentUid]) {		
		originalValue = nil;
		baseValue = nil;
		endValue = nil;
		currentValue = 0;
		isPlaying = NO;
		startedTime = -1;
		endingTime = -1;
		currentRepetition = 0;
		currentDur = -1;
		
		// get control and attribute to animate
		AMAnimation *anim = (AMAnimation *)parent;
		control = [[anim getControlParent] createControlWithName:[anim getControlName]];
		attributeType = [control attributeCode:[anim getAttributeName]];
	}
	return self;
}

- (id)start
{
	if (!isPlaying) {
		AMAnimation *anim = (AMAnimation *)parent;
		
		startedTime = [AMSoundManager getScheduler].currentTime;
		isPlaying = YES;
		currentRepetition = 0;
		
		// init original value if unset
		if (!originalValue) {
			originalValue = [control valueOfAttribute:attributeType];
		}
		
		// init base value if unset
		if (!baseValue) {
			if (anim.from)
				baseValue = [[NSNumber alloc] initWithFloat:[anim.from calcValue]];
			else
				baseValue = [originalValue retain];
		}
		
		// try to init end value if unset
		if (!endValue) {
			if (anim.by)
				endValue = [[NSNumber alloc] initWithFloat:[baseValue floatValue] + [anim.by calcValue]];
			else if (anim.to)
				endValue = [[NSNumber alloc] initWithFloat:[anim.to calcValue]];
		}
		
		// calculate the duration of the animation
		int tempo = [(AMTempoControl *)[self getClosestControlWithName:AM_TEMPO_CONTROL] tempo];
		double quantizer = [((AMAnimation *)parent).quantize calcTimeWithTempo:tempo];
		BOOL isQTempoBased = [((AMAnimation *)parent).quantize isTimeTempoBased];
		double end;
		endingTime = -1;

		if (anim.repeatDur) {
			endingTime = startedTime + [anim.repeatDur calcTimeWithTempo:tempo quantizer:quantizer isQuantizerTempoBased:isQTempoBased];
		}

		if (anim.dur) {
			currentDur = [anim.dur calcTimeWithTempo:tempo quantizer:quantizer isQuantizerTempoBased:isQTempoBased];
			
			if(anim.repeatCount >= 1) {
				end = startedTime + anim.repeatCount * currentDur;
				if (endingTime == -1 || end < endingTime)
					endingTime = end;
			}
		}
		
		// set first value
		[self animate:startedTime];
		
		// send notification
		NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
		event = [event stringByAppendingString:AM_PLAYABLE_BEGIN_SUFFIX];
		
		[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
		
		// TODO:send also instance event
		
	}
	
	return self;
}

- (id)stop
{
	if (isPlaying) {
		AMAnimation *anim = (AMAnimation *)parent;
		
		isPlaying = NO;
		startedTime	= -1;
		[baseValue release];
		baseValue = nil;
		
		if (anim.fill == AM_FIL_FREEZE && endValue) {
			[control setValue:endValue forAttribute:attributeType];
		} else if (originalValue) {
			[control setValue:originalValue forAttribute:attributeType];
		}
		
		[originalValue release];
		originalValue = nil;
		[endValue release];
		endValue = nil;
		
		// send notification
		NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
		event = [event stringByAppendingString:AM_PLAYABLE_END_SUFFIX];
		
		[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
		
		// TODO:send also instance event		
		
	}
	
	return nil;
}

- (BOOL)isPlaying
{
	return isPlaying;
}

- (double)getDuration
{
	if (startedTime && endingTime) 
		return endingTime - startedTime;
	return -1;
}

// private method
- (NSNumber *)getValueFromListWithTime:(double)_currentTime 
{
	AMAnimation *anim = (AMAnimation *)parent;
	NSNumber *value = nil;
	
	if (anim.values && [anim.values count] > 0) {
		if (currentDur) {
			float timeIndex = ((float)([anim.values count] - 1) * (_currentTime - startedTime)) / currentDur;
			int currIndex = (int)timeIndex;
			float baseVal = [(NSNumber *)[anim.values objectAtIndex:currIndex] floatValue];
			
			// calc linear interpolation between values
			if (currIndex + 1 < [anim.values count]) {
				float diffVal = baseVal - [(NSNumber *)[anim.values objectAtIndex:currIndex + 1] floatValue];
				value = [NSNumber numberWithFloat:(baseVal + (diffVal * (timeIndex - (float)currIndex)))];
			} else {
				value = [NSNumber numberWithFloat:baseVal];
			}
		} else {
			value = [anim.values objectAtIndex:0];
		}
	}
	
	return value;
}

- (void)animate:(double)_currentTime
{
	if (isPlaying) {
		AMAnimation *anim = (AMAnimation *)parent;
		
		// update last repetition started time
		if (anim.dur) {
			while (_currentTime - startedTime > currentDur) {
				startedTime += currentDur;
				++currentRepetition;
				
				// update values for cumulative repetition if needed
				if (anim.accumulation == AM_ACC_SUM && baseValue && endValue) {
					float base = [baseValue floatValue];
					float end = [endValue floatValue];
					
					[baseValue release];
					baseValue = endValue;
					endValue = [[NSNumber alloc] initWithFloat:(end + end - base)];
				}
				
				NSDictionary *val = [NSDictionary dictionaryWithObject:[NSNumber numberWithFloat:currentRepetition] forKey:AM_EVENT_OBJECT_DEFAULT_KEY];
				NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
				event = [event stringByAppendingString:AM_PLAYABLE_REPEAT_SUFFIX];
				
				[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease] userInfo:val];
				
				// TODO:send also instance event						
				
			}
		}
		
		// check if animation has ended
		if (endingTime && _currentTime >= endingTime) {
			[self stop];
		} else  if (baseValue) {
			// try to determinate the current value from the value list
			NSNumber *listValue = [self getValueFromListWithTime:_currentTime];
		
			if (listValue) {
				currentValue = [listValue floatValue];
			} else {
				// use from/to/by to determinate current animation value
				if (endValue && currentDur) {
					// calculate the new animation value if everything is correctly defined
					currentValue = [baseValue floatValue] + (([endValue floatValue] - [baseValue floatValue]) * (_currentTime - startedTime) / currentDur);
				} else {
					// if ending value or ending time of the animation is undefined, use the base value
					currentValue = [baseValue floatValue];
				}
			}

			[control setValue:[NSNumber numberWithFloat:currentValue] forAttribute:attributeType];
		}
	}
}

-(void)dealloc 
{
	// release vars
	[baseValue release];
	[originalValue release];
	[endValue release];

	
	[super dealloc];
}

@end
