/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMCueInstance.mm
//  aria
//
//  Created by Yohan Lasorsa on 03/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMCueInstance.h"


@implementation AMCueInstance

@synthesize releaseTimeout;

- (id)initWithParent:(id)_parent parentUid:(UID)_parentUid
{
	if (self = [super initWithParent:_parent parentUid:_parentUid]) {		
		if (parent != nil) {
			currentChunk = 0;
			chunksPlaying = 0;
			
			if ([(AMCue *)parent loopCount] == -1)
				remainingLoops = -1;
			else
				remainingLoops = [(AMCue *)parent loopCount] - 1;

			[self updateNextChunk];
			chkInstance1 = nil;
			chkInstance2 = nil;
			controls = nil;
		}
	}
	return self;
}

- (int)updateNextChunk
{
	nextChunk = currentChunk + 1;
	if (nextChunk >= [[(AMCue *)parent getChunks] count])
		nextChunk = 0;
	return nextChunk;
}

- (id)start
{
	// check if it's already playing
	if ([self isPlaying]) {
		// check what to do in that case
		AMRetriggerType retrig = [(AMCue *)parent restartOnRetrigger];
		
		// TODO: check for AUTO and instance count
		if (retrig == AM_RT_ALWAYS) {
			[self stop];
		} else if (retrig == AM_RT_NEVER) {
			return self;
		}
	}
	
	// start the cue
	NSArray *chunks = [(AMCue *)parent getChunks];
	
	if ([(AMCue *)parent loopCount] == -1)
		remainingLoops = -1;
	else
		remainingLoops = [(AMCue *)parent loopCount] - 1;
	
	chunksPlaying = 0;
			
	if (currentChunk < [chunks count]) {
		chkInstance1 = [(AMChunk *)[chunks objectAtIndex:currentChunk] startFromParentUid:parentUid listener:self];
	}

	[AMSoundManager removeInstanceFromReleasePool:self];	// if it was in there
	
	return self;
}

- (id)stop
{
	[chkInstance1 stop];
	[chkInstance2 stop];
	
	[AMSoundManager addInstanceToReleasePool:self];	
	return self;
}

- (BOOL)isPlaying
{
	if (chkInstance1 != nil)
		return [chkInstance1 isPlaying];
	else if (chkInstance2 != nil)
		return [chkInstance2 isPlaying];
	else
		return NO;
}

- (double)getDuration
{
	// TODO: duration for chunk not implemented for now.
	return -1;
}

- (NSSet *)getControls
{
	return [NSSet setWithObjects:AM_MIX_2D_CONTROL, AM_MIX_3D_CONTROL, nil];
}

- (AMControl *)getControlWithName:(NSString *)_name
{
	AMControl *control = [controls objectForKey:_name];
	
	// if the control has not been instanciated, check if the parent has it
	if (control == nil) {
		control = [(AMCue *)parent getControlWithName:_name];
		
		// if the control is valid, make a copy and own it
		if (control != nil) {
			control = [control copy];
			control.parent = self;
			
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];			
			[controls setObject:control forKey:_name];
		}
	} else {
		// keeps the object valid if the sound instance gets killed.
		[[control retain] autorelease];
	}
	
	return control;
}

- (AMControl *)createControlWithName:(NSString *)_name
{
	AMControl *control = [self getControlWithName:_name];
	
	// if the control has not been instanciated
	if (control == nil) {
		control = [(AMCue *)parent getControlWithName:_name];

		// if the control is valid, make a copy and own it
		if (control != nil) {
			control = [control copy];
			control.parent = self;
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];			
			[controls setObject:control forKey:_name];
		} else {
			control = [AMControl controlWithName:_name controllable:self];
			if (control != nil) {
				if (controls == nil)
					controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];			
				[controls setObject:control forKey:_name];
			}
		}
	}
	
	return control;
}

- (void)chunkUpdate:(AMChunk *)_chunk eventType:(int)_type eventData:(void *)_data
{
	switch (_type) {
		case AM_CHK_STARTED:
		{
			AM_LOG_CORE(@"[ID:%lld, PID:%lld] Chunk %d started.", uid, parentUid, currentChunk+1);
			
			[(id <AMCueListener>)listener cueUpdate:parent eventType:AM_CUE_STARTED eventData:NULL];
			
			// send notification
			NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
			event = [event stringByAppendingString:AM_PLAYABLE_BEGIN_SUFFIX];
			
			[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
			
			// TODO:send also instance event
			
			chunksPlaying++;
			break;
		}
		case AM_CHK_STOPPED:
		{
			[(id <AMCueListener>)listener cueUpdate:parent eventType:AM_CUE_STOPPED eventData:NULL];
			
			// send notification
			NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
			event = [event stringByAppendingString:AM_PLAYABLE_END_SUFFIX];
			
			[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
			
			// TODO:send also instance event
			
			chunksPlaying--;
			break;
		}
		case AM_CHK_ENDED:
		{
			AM_LOG_CORE(@"[ID:%lld, PID:%lld] Chunk %d ended.", uid, parentUid, currentChunk+1);
			chunksPlaying--;

			if (nextChunk == 0) {
				if (remainingLoops > 0) {
					remainingLoops--;
					
					// case of a crossfade transition
					if (chunksPlaying > 0) {
						chkInstance1 = chkInstance2;
						chkInstance2 = nil;
						
					} else {
						// start next chunk
						currentChunk = nextChunk;
						chkInstance1 = [(AMChunk *)[[parent getChunks] objectAtIndex:nextChunk] startFromParentUid:parentUid listener: self];
						[self updateNextChunk];
						
						// send notification
						NSDictionary *val = [NSDictionary dictionaryWithObject:[NSNumber numberWithFloat:([(AMCue *)parent loopCount] - 1 - remainingLoops)] forKey:AM_EVENT_OBJECT_DEFAULT_KEY];
						NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
						event = [event stringByAppendingString:AM_PLAYABLE_REPEAT_SUFFIX];
						
						[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease] userInfo:val];
						
						// TODO:send also instance event						
					}
					
				} else if (remainingLoops <= -1) {
					remainingLoops--;
					
					// case of a crossfade transition
					if (chunksPlaying > 0) {
						chkInstance1 = chkInstance2;
						chkInstance2 = nil;

					} else {
						// start next chunk
						currentChunk = nextChunk;
						chkInstance1 = [(AMChunk *)[[parent getChunks] objectAtIndex:nextChunk] startFromParentUid:parentUid listener: self];
						[self updateNextChunk];

						// send notification
						NSDictionary *val = [NSDictionary dictionaryWithObject:[NSNumber numberWithFloat:(-remainingLoops - 1)] forKey:AM_EVENT_OBJECT_DEFAULT_KEY];
						NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
						event = [event stringByAppendingString:AM_PLAYABLE_REPEAT_SUFFIX];
						
						[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease] userInfo:val];
						
						// TODO:send also instance event						
					}
					
				} else if (chunksPlaying == 0) {
					[(id <AMCueListener>)listener cueUpdate:parent eventType:AM_CUE_ENDED eventData:NULL];

					// send notification
					NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];
					event = [event stringByAppendingString:AM_PLAYABLE_END_SUFFIX];
					
					[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
					
					// TODO:send also instance event
					
					AM_LOG_CORE(@"[ID:%lld, PID:%lld] Ended, remaining loops: %d.", uid, parentUid, remainingLoops);
					
					[AMSoundManager addInstanceToReleasePool:self];
				}
			} else {
				
				// play next chunk
				
				//TODO: send notification?
				
				// case of a crossfade transition
				if (chunksPlaying > 0) {
					chkInstance1 = chkInstance2;
					chkInstance2 = nil;					
				} else {
					// start next chunk
					currentChunk = nextChunk;
					chkInstance1 = [(AMChunk *)[[parent getChunks] objectAtIndex:nextChunk] startFromParentUid:parentUid listener: self];
					[self updateNextChunk];
				}
				
				
			}
			break;
		}
		default:
			break;
	}
}

- (AMChunkInstance *)getChunkInstance1
{
	return chkInstance1;
}

- (AMChunkInstance *)getChunkInstance2
{
	return chkInstance2;
}

-(void)dealloc 
{	
	// stop chunk instances
//	[chkInstance1 stop];	// can't do it, because we're unsure if these vars are safe at this point (ie BAD_ACCESS may occur)
//	[chkInstance2 stop];

	AM_LOG_CORE(@"[ID:%lld, PID:%lld] Released.", uid, parentUid);
	
	// release vars
	[controls release];

	[super dealloc];
}

@end
