/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSoundInstance.mm
//  aria
//
//  Created by Yohan Lasorsa on 03/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMSoundInstance.h"
#import "AMMix3DControl.h"
#import "AMMix2DControl.h"
#import "AMSoundControl.h"
#import "AMCue.h"


// prototypes for internal functions
FMOD_RESULT F_CALLBACK channelCallback(FMOD_CHANNEL *channel, FMOD_CHANNEL_CALLBACKTYPE type, void *commanddata1, void *commanddata2);

@implementation AMSoundInstance

@synthesize remainingLoops;
@synthesize stopped;

/*
- (id)initWithParent:(id)_parent
{
	return [self initWithParent:_parent id:GenerateUniqueID()];
}*/

- (id)initWithParent:(id)_parent parentUid:(UID)_parentUid
{
	if (self = [super initWithParent:_parent parentUid:_parentUid]) {		
		if (parent != nil) {
			// init variables
			remainingLoops = [(AMSound *)parent loopCount] - 1;
			stopped = NO;
			controls = nil;
			channel = NULL;
		}
	}
	return self;
}

- (NSSet *)getControls
{
	return [NSSet setWithObjects:AM_MIX_2D_CONTROL, AM_SOUND_CONTROL, nil];
}

- (AMControl *)getControlWithName:(NSString *)_name
{
	AMControl *control = [controls objectForKey:_name];
	
	// if the control has not been instanciated, check if the parent has it
	if (control == nil) {
		control = [(AMSound *)parent getControlWithName:_name];
		
		// if the control is valid, make a copy and own it
		if (control != nil) {
			control = [control copy];
			control.parent = self;
			
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];			
			[controls setObject:control forKey:_name];
		}
	} else {
		// keeps the object valid if the sound instance gets killed.
		[[control retain] autorelease];
	}
	
	return control;
}

- (AMControl *)createControlWithName:(NSString *)_name
{
	AMControl *control = [self getControlWithName:_name];
	
	// if the control has not been instanciated
	if (control == nil) {
		control = [(AMSound *)parent getControlWithName:_name];
		
		// if the control is valid, make a copy and own it
		if (control != nil) {
			control = [control copy];
			control.parent = self;
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];			
			[controls setObject:control forKey:_name];
		} else {
			control = [AMControl controlWithName:_name controllable:self];
			if (control != nil) {
				if (controls == nil)
					controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];			
				[controls setObject:control forKey:_name];
			}
		}
	}
	
	return control;
}

- (void)startSound
{
	// play sound
	FMOD_RESULT result;
	result = [AMSoundManager getFMODSystem]->playSound(FMOD_CHANNEL_FREE, [(AMSound *)parent getFMODSound], true, &channel);
	ERRCHECK(result);
	
	// apply 3D parameters
	AMCueInstance *parentCueInstance = [[(AMSound *)parent getParentCue] getCueInstanceWithParentUID:parentUid];
	AMMix3DControl *mix3d = (AMMix3DControl *)[parentCueInstance getControlWithName:AM_MIX_3D_CONTROL];
	
	if (mix3d == nil) {
		// ensure 2D mode is activated
		channel->setMode(FMOD_2D);
	} else {
		[mix3d applySettingsToInstance:self];
	}
	
	// set sound mix parameters
	AMMix2DControl *mix2d = (AMMix2DControl *)[(AMSound *)parent getControlWithName:AM_MIX_2D_CONTROL];
	
	if (mix2d) {
		mix2d = [mix2d copy];
		mix2d.parent = self;
		[controls setObject:mix2d forKey:AM_MIX_2D_CONTROL];
		
		// apply parameters
		[mix2d applySettingsToInstance:self];
	} else {
		// if control is defined in the parent sound, try to get it from the parent cue instance 
		mix2d = (AMMix2DControl *)[parentCueInstance getControlWithName:AM_MIX_2D_CONTROL];
		[mix2d applySettingsToInstance:self];
	}

	
	// set sound specific parameters
	AMSoundControl *sndControl = (AMSoundControl *)[(AMSound *)parent getControlWithName:AM_SOUND_CONTROL];
	
	if (sndControl) {
		sndControl = [sndControl copy];
		sndControl.parent = self;
		[controls setObject:sndControl forKey:AM_SOUND_CONTROL];
		
		// apply parameters
		[sndControl applySettingsToInstance:self];
	}
	
	// make the right DSP connections if needed
	NSArray *cgroups = *[[(AMSound *)self.parent getParentCue] getFMODChannelGroups];
	
	if (cgroups != nil) {
		// make the first, simple connection
		channel->setChannelGroup((FMOD::ChannelGroup *)[(NSValue *)[cgroups objectAtIndex:0] pointerValue]);
		
		if ([cgroups count] > 1) {
			// make the connection with other channel groups' head DSP
			FMOD::DSP *chanDSPHead;
			FMOD::DSP *cgroupDSPHead;
			FMOD::ChannelGroup *cgroup;
			
			channel->getDSPHead(&chanDSPHead);
			
			for (int i=1; i < [cgroups count]; i++) {
				cgroup = (FMOD::ChannelGroup *)[(NSValue *)[cgroups objectAtIndex:i] pointerValue];
				cgroup->getDSPHead(&cgroupDSPHead);
				cgroupDSPHead->getInput(0, &cgroupDSPHead, NULL); // TODO: check unneeded?
				cgroupDSPHead->addInput(chanDSPHead, NULL);
			}
		}
	}
	
	// set channel callback
	channel->setUserData(self);
	channel->setCallback(channelCallback);
	
	// start the sound
	channel->setPaused(false);	
}

- (id)start
{
	remainingLoops = [(AMSound *)parent loopCount] - 1;
	
	// configure and start sound
	[self startSound];
	
	// send started event
	[(id <AMSoundListener>)listener soundUpdate:parent eventType:AM_SND_STARTED eventData:NULL];
	
	// send global notification
	// TODO:
	//[[NSNotificationCenter defaultCenter] postNotificationName:idf object:self];
	
	AM_LOG_CORE(@"[ID:%lld, PID:%lld] Started.", uid, parentUid);
	
	return self;
}

- (id)stop
{
	stopped = YES;
	channel->stop();
	[self kill];
	
	// send stopped event
	[(id <AMSoundListener>)listener soundUpdate:parent eventType:AM_SND_STOPPED eventData:NULL];
	
	//TODO: send global notification

	return nil;
}

- (BOOL)isPlaying
{
	BOOL playing = false;
	
	if (channel) {
		ERRCHECK(channel->isPlaying((bool *)&playing));
	}
	
	return playing;
}

- (double)getDuration
{
	double dur = -1;
	
	if (parent) {
		dur = [(AMSound *)parent getDuration];
	}
	
	return dur;
}
	
- (void)kill
{
	if (channel) {
		channel->stop();
	}
	
	[super kill];
}

- (FMOD::Channel **)getFMODChannelPtr
{
	return &channel;
}

-(void)dealloc 
{
	AM_LOG_CORE(@"[ID:%lld, PID:%lld] Released.", uid, parentUid);
	
	// remove channel callback
//	channel->setCallback(NULL);
//	channel->setUserData(NULL);
	
	// release vars
	[controls release];
	
	// removes the DSP connections if needed
	NSArray *cgroups = *[[(AMSound *)self.parent getParentCue] getFMODChannelGroups];
	
	if (cgroups != nil) {
		FMOD::ChannelGroup *cgroup;
		
		[AMSoundManager getFMODSystem]->getMasterChannelGroup(&cgroup);
		channel->setChannelGroup(cgroup);
		
		if ([cgroups count] > 1) {
			// removes other connections
			FMOD::DSP *chanDSPHead;
			FMOD::DSP *cgroupDSPHead;
			
			channel->getDSPHead(&chanDSPHead);
			
			for (int i=1; i < [cgroups count]; i++) {
				cgroup = (FMOD::ChannelGroup *)[(NSValue *)[cgroups objectAtIndex:i] pointerValue];
				cgroup->getDSPHead(&cgroupDSPHead);
				cgroupDSPHead->getInput(0, &cgroupDSPHead, NULL); // TODO: check unneeded?
				cgroupDSPHead->disconnectFrom(chanDSPHead);
			}
		}
	}	
	
	[super dealloc];
}

@end


FMOD_RESULT F_CALLBACK channelCallback(FMOD_CHANNEL *channel, FMOD_CHANNEL_CALLBACKTYPE type, void *commanddata1, void *commanddata2)
{	
	if (type == FMOD_CHANNEL_CALLBACKTYPE_END) {
		AMSoundInstance *sInstance;
		((FMOD::Channel *)channel)->getUserData((void **)&sInstance);
		int remainingLoops = sInstance.remainingLoops;
		
		if (!sInstance.stopped) {
			
			if (remainingLoops <= -1) {
				// play sound
				[sInstance startSound];
//				FMOD_RESULT result;
//				result = [AMSoundManager getFMODSystem]->playSound(FMOD_CHANNEL_REUSE, [(AMSound *)[sInstance parent] getFMODSound], false, [sInstance getFMODChannelPtr]);
//				ERRCHECK(result);		
			} else if (remainingLoops > 0) {
				sInstance.remainingLoops--;
				
				// play sound
				[sInstance startSound];
//				FMOD_RESULT result;
//				result = [AMSoundManager getFMODSystem]->playSound(FMOD_CHANNEL_REUSE, [(AMSound *)[sInstance parent] getFMODSound], false, [sInstance getFMODChannelPtr]);
//				ERRCHECK(result);
			} else {				
				
				// send ended event
				[(id <AMSoundListener>)[sInstance listener] soundUpdate:[sInstance parent] eventType:AM_SND_ENDED eventData:NULL];
				
				//TODO: send global notification
				
				[sInstance kill];
				
				// TODO: is remove channel callback needed?
				//			((FMOD::Channel *)channel)->setCallback(NULL);		
			}
		}
	}
	
	return FMOD_OK;
}
