/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  a2ml.h
//  aria
//
//  Created by Yohan Lasorsa on 5/27/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "fmod.hpp"
#import "fmod_errors.h"
#import <Foundation/Foundation.h>


/* Debug defines */

#define AM_DEBUG_PARSER		1	// Show parser debug information.
#define	AM_DEBUG_SCHEDULER	1	// Show scheduler debug information.
#define AM_DEBUG_ANIMATION	1	// Show animation debug information.
#define AM_DEBUG_CORE		1	// Show core objects debug information.
#define AM_FMOD_CHECKS		1	// Active FMOD functions error checking.

#ifdef DEBUG
#define	AM_LOG(f, ...)				printf("%s\n", [[NSString stringWithFormat:(@"ARIA: %-16s : " f), [NSStringFromClass([self class]) cStringUsingEncoding:NSUTF8StringEncoding], ##__VA_ARGS__] cStringUsingEncoding:NSUTF8StringEncoding])
#define	AM_LOGC(f, ...)				printf("%s\n", [[NSString stringWithFormat:(@"ARIA: %-16s : " f), __FUNCTION__, ##__VA_ARGS__] cStringUsingEncoding:NSUTF8StringEncoding])
#else
#define	AM_LOG(...)	
#define	AM_LOGC(f, ...)				
#endif

#ifdef AM_DEBUG_CORE
#define	AM_LOG_CORE(f, ...)			AM_LOG(f, ##__VA_ARGS__)
#else
#define	AM_LOG_CORE(...)	
#endif

#ifdef AM_DEBUG_PARSER
#define	AM_LOG_PARSER(f, ...)		AM_LOG(f, ##__VA_ARGS__)
#else
#define	AM_LOG_PARSER(...)	
#endif

#ifdef AM_DEBUG_SCHEDULER
#define	AM_LOG_SCHEDULER(f, ...)	AM_LOG(f, ##__VA_ARGS__)
#else
#define	AM_LOG_SCHEDULER(...)	
#endif

#ifdef AM_DEBUG_ANIMATION
#define	AM_LOG_ANIMATION(f, ...)	AM_LOG(f, ##__VA_ARGS__)
#else
#define	AM_LOG_ANIMATION(...)	
#endif

/* A2ML defines */

#define AM_CONTROLS_INITIAL_CAPACITY	1		/**< The initial capacity for controls dictonaries. */
#define AM_DEFAULT_TEMPO				120		/**< Default tempo value. */
#define	AM_TEMPO_RESOLUTION				384		/**< The tempo resolution, ie the number of ticks per beat. */
#define AM_TIME_SEP_INTERVAL			1.0		/**< The time separation interval for the scheduler time manager, in seconds. */
#define UNDEFINED_UID					0		/**< Equivalent to NULL for an UID. */


/* Helper defines */

#define ARC4RANDOM_MAX		0x100000000LL		/* arc4random() function maximum generated integer. */
#define RAND(x)				(((double)arc4random() / ARC4RANDOM_MAX) * (x))			/* Generates a random number between 0 and x. */
#define RAND_INT(x)			((int)(floorf(((double)arc4random() / ARC4RANDOM_MAX) * (x))))	/* Generates a random integer number between 0 and x. */

/* String constants */

#define AM_PLAYABLE_BEGIN_SUFFIX		@".begin"
#define AM_PLAYABLE_END_SUFFIX			@".end"
#define AM_PLAYABLE_REPEAT_SUFFIX		@".repeat"
#define AM_EVENT_OBJECT_DEFAULT_KEY		@"value"

/* Custom types */

/** A unique object identifier. */
typedef long long UID;

/** A 3-Dimension vector. */
typedef struct {
	float  x;
	float  y;
	float  z;
} AMVec3;

/* Enumerations */

/**
 The different pick methods available.
 */
enum AMPickType {
	AM_PT_ORDERED = 0,
	AM_PT_RANDOM,
	AM_PT_OMIT_MOST_RECENT,
	AM_PT_REVERSED,
	AM_PT_FIXED
};

/**
 The different fade in methods available.
 */
enum AMFadeInType {
	AM_FIT_NONE = 0,
	AM_FIT_SIMPLEFADE
};

/**
 The different fade out methods available.
 */
enum AMFadeOutType {
	AM_FOT_NONE = 0,
	AM_FOT_SIMPLEFADE,
	AM_FOT_CROSSFADE
};

/**
 The different retrigger options.
 */
enum AMRetriggerType {
	AM_RT_AUTO = 0,
	AM_RT_ALWAYS,
	AM_RT_NEVER
};

/**
 The different schedulable action types.
 */
enum AMActionType {
	AM_ACT_NONE = 0,
	AM_ACT_START,
	AM_ACT_STOP,
	AM_ACT_SET,
	AM_ACT_ENABLE,
	AM_ACT_DISABLE,
	AM_ACT_SET_ACTIVE
};

/**
 The different possible fill types for animations.
 */
enum AMFillType {
	AM_FIL_REMOVE = 0,
	AM_FIL_FREEZE,
};

/**
 The different accumulation types for animations.
 */
enum AMAccumulationType {
	AM_ACC_NONE = 0,
	AM_ACC_SUM,
};


/*  Helper functions*/

void ERRCHECK(FMOD_RESULT result);		/**< Checks for FMOD errors. */
long long GenerateUniqueID();			/**< Generates a unique ID. */
