/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMParser.mm
//  aria
//
//  Created by Yohan Lasorsa on 01/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMParser.h"


// Constants
#define LIST_SEPARATOR	@" "

const NSArray *enumPickType = [[NSArray alloc] initWithObjects:@"ordered", @"random", @"exclusiveRandom", @"reversed", @"fixed", nil];
const NSArray *enumFadeInType = [[NSArray alloc] initWithObjects:@"none", @"simplefade", nil];
const NSArray *enumFadeOutType = [[NSArray alloc] initWithObjects:@"none", @"simplefade", @"crossfade", nil];
const NSArray *enumRetriggerType = [[NSArray alloc] initWithObjects:@"auto", @"always", @"never", nil];
const NSArray *enumFillType = [[NSArray alloc] initWithObjects:@"remove", @"freewe", nil];
const NSArray *enumAccumulationType = [[NSArray alloc] initWithObjects:@"none", @"sum", nil];


// Private methods
@interface AMParser()

/* Checks if the given string ID is unique and valid. */
- (BOOL)checkID:(NSString *) _uid;

/* Check if the given path is valid (ie the target file exists). */
- (BOOL)checkPath:(NSString *)_path;

/* Parses a Time/Event list from a string. */
- (void)parseEventOrTimeList:(NSString *)_str playable:(id<AMPlayable>)_playable action:(AMActionType)_action;

/* Parses a Value element from a string. */
- (AMValue *)parseValue:(NSString *)_str;

/* Parses a Time element from a string. */
- (AMTime *)parseTime:(NSString *)_str;

/* Splits a list of multiple space-separated string elements into an array. */
- (NSArray *)parseList:(NSString *)_str;

/* Parses a list of doube values into an array of NSNumber objects. */
- (NSArray *)parseDoubleList:(NSString *)_str;

/* Parses a list of XML object ID references into an array of the corresponding A2ML objects. */
- (NSArray *)parseReferenceList:(NSString *)_str;

/* Parses a string event, and gets its optional delay (if applicable) in _delay. */
- (NSString *)parseEvent:(NSString *)_str delay:(AMTime **)_delay;

/* Parses a simple integer value, providing the default value if the parsing fails. */
- (int)parseInt:(NSString *) _str withDefaultValue:(int) _defaultVal;

/* Parses a simple double value, providing the default value if the parsing fails. */
- (int)parseDouble:(NSString *)_str withDefaultValue:(double)_defaultVal;

/* Parses a simple boolean value, providing the default value if the parsaing fails. */
- (BOOL)parseBoolean:(NSString *)_str withDefaultValue:(BOOL) _defaultVal;

/* Parses an enumeration, providing the default value and the enum string array. */
- (int)parseEnum:(NSString *)_str withDefaultValue:(int)_defaultVal enumStrings:(NSArray *)_enumStrings;

@end


@implementation AMParser

- (id)init
{
	if (self = [super init]) {
		lastControllable = nil;
		lastPlayable = nil;
		lastCue = nil;
		lastChunk = nil;
		lastSound = nil;
		inMasterSection = NO;
	}
	return self;
}

- (void)dealloc
{
	[super dealloc];
}

- (void)parseA2MLFileAtURL:(NSString *)_URL parseError:(NSError **)_error
{	
	// Gets the base component of the path
	basePath = [[_URL substringToIndex:([_URL length] - [[_URL lastPathComponent] length])] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
	if (![basePath hasSuffix:@"/"] && ![basePath isEqualToString:@""])
		basePath = [basePath stringByAppendingString:@"/"];
	
	AM_LOG_PARSER(@"Using URL base: '%@'", basePath);
	
    // Convert the path to a proper NSURL
	NSData *xmlData = [NSData dataWithContentsOfFile:_URL];
	NSXMLParser *parser = [[NSXMLParser alloc] initWithData:xmlData];
	
    // Set self as the delegate of the parser so that it will receive the parser delegate methods callbacks
    [parser setDelegate:self];
	
    // Configure the parser
    [parser setShouldProcessNamespaces:NO];
    [parser setShouldReportNamespacePrefixes:NO];
    [parser setShouldResolveExternalEntities:NO];
    
	// Parse the document
	AM_LOG_PARSER(@"Parsing URL %@", _URL);
    [parser parse];
    
    NSError *parseError = [parser parserError];
    if (parseError && _error) {
		AM_LOG_PARSER(@"An error occured while parsing!");
        *_error = parseError;
    }
    
    [parser release];
}

- (void)parserDidStartDocument:(NSXMLParser *)parser
{
	
	//NSString *path = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"myFile.xml"];
	
	//	DISPLOG(@"document start.");
	
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
    if (qName) {
        elementName = qName;
    }
	
	//	[self parseEventOrTimeList:@"test1 test2.test1    test3.ok.baba  +  2 test4-3.5ms test5 + rand(2 , 3.0)B test6+rand(1,2,3.7)s" playable:nil action:AM_ACT_START];

	if ([elementName isEqualToString:@"masterSection"]) {
		
		inMasterSection = YES;
		
    } else if ([elementName isEqualToString:@"section"]) {
		
		AM_LOG_PARSER(@"Found Section.");
		
		// retrieve section attributes
		NSString *idf = [attributeDict valueForKey:@"id"];
		NSString *cues = [attributeDict valueForKey:@"cues"];
		NSString *volume = [attributeDict valueForKey:@"volume"];
		NSString *pan = [attributeDict valueForKey:@"pan"];
		
		// create the section if the ID is valid
		if ([self checkID:idf]) {
			
			AM_LOG_PARSER(@"Section ID '%@' is valid.", idf);
			
			AMSection *section = [[AMSection alloc] init];
			[section autorelease];
			
			lastControllable = section;
			
			// register ID
			[AMSoundManager addKeyUID:[section getUID] forID:idf];
			[AMSoundManager addKeyID:idf forObject:section];			// TODO: store it elsewhere?			
			
			// configure the section
			NSArray *refList = [self parseReferenceList:cues];			
			for (AMCue *cue in refList) {
				[section addCue:cue];
				AM_LOG_PARSER(@"Added cue '%lld' to section '%@'.", [cue getUID], idf);
			}
			
			// configure mix controls
			if (volume != nil || pan != nil) {
				AMMix2DControl *mix2DControl = (AMMix2DControl *)[section createControlWithName:AM_MIX_2D_CONTROL]; 
				mix2DControl.volume = [self parseInt:volume withDefaultValue:100];
				mix2DControl.pan = [self parseInt:pan withDefaultValue:0];
				AM_LOG_PARSER(@"Mix 2D parameters defined for section '%@'.", idf);
			}
			
			// TODO: store section in some array? SOundManager?
		}
		
		
    } else if ([elementName isEqualToString:@"cue"]) {
		
		AM_LOG_PARSER(@"Found Cue.");
		
		// retrieve cue attributes
		NSString *idf = [attributeDict valueForKey:@"id"];
		NSString *begin = [attributeDict valueForKey:@"begin"];
		NSString *dur = [attributeDict valueForKey:@"dur"];
		NSString *end = [attributeDict valueForKey:@"end"];
		NSString *quantize = [attributeDict valueForKey:@"quantize"];
		NSString *loopCount = [attributeDict valueForKey:@"loopCount"];
		NSString *priority = [attributeDict valueForKey:@"priority"];
		NSString *restartOnRetrigger = [attributeDict valueForKey:@"restartOnRetrigger"];
		NSString *stream = [attributeDict valueForKey:@"stream"];
		
		// create the cue if the ID is valid
		if ([self checkID:idf]) {
			
			AM_LOG_PARSER(@"Cue ID '%@' is valid.", idf);
			
			AMCue *cue = [[AMCue alloc] init];
			[cue autorelease];
			
			lastCue = cue;
			lastPlayable = cue;
			lastControllable = cue;
			
			// register ID
			[AMSoundManager addKeyUID:[cue getUID] forID:idf];
			[AMSoundManager addKeyID:idf forObject:cue];
			
			// configure the cue
			[self parseEventOrTimeList:begin playable:cue action:AM_ACT_START];
			[cue setDur:[self parseTime:dur]];
			[self parseEventOrTimeList:end playable:cue action:AM_ACT_STOP];
			[cue setQuantize:[self parseTime:quantize]];
			[cue setLoopCount:[self parseInt:loopCount withDefaultValue:1]];
			[cue setPriority:[self parseInt:priority withDefaultValue:1]];
			[cue setRestartOnRetrigger:(AMRetriggerType)[self parseEnum:restartOnRetrigger withDefaultValue:AM_RT_AUTO enumStrings:enumRetriggerType]];
			[cue setStream:[self parseBoolean:stream withDefaultValue:NO]];
			
			// TODO: store cue in some array? SOundManager?
		}
		
    } else if ([elementName isEqualToString:@"chunk"]) {
		
		AM_LOG_PARSER(@"Found Chunk.");
		
		// retrieve chunk attributes
		NSString *idf = [attributeDict valueForKey:@"id"];
		NSString *dur = [attributeDict valueForKey:@"dur"];
		NSString *pick = [attributeDict valueForKey:@"pick"];
		NSString *fadeInType = [attributeDict valueForKey:@"fadeInType"];
		NSString *fadeOutType = [attributeDict valueForKey:@"fadeOutType"];
		NSString *fadeInDur = [attributeDict valueForKey:@"fadeInDur"];
		NSString *fadeOutDur = [attributeDict valueForKey:@"fadeOutDur"];
		NSString *activeSound = [attributeDict valueForKey:@"activeSound"];
		
		// create the chunk
		AMChunk *chunk = [[AMChunk alloc] initWithParent:lastCue];
		[chunk autorelease];
		
		lastChunk = chunk;
		lastControllable = chunk;
		
		// register ID if it's valid
		if ([self checkID:idf]) {
			[AMSoundManager addKeyUID:[chunk getUID] forID:idf];
		}
		
		// configure the chunk
		[chunk setDur:[self parseTime:dur]];
		[chunk setPick:(AMPickType)[self parseEnum:pick withDefaultValue:AM_PT_ORDERED enumStrings:enumPickType]];
		[chunk setFadeInType:(AMFadeInType)[self parseEnum:fadeInType withDefaultValue:AM_FIT_NONE enumStrings:enumFadeInType]];
		[chunk setFadeOutType:(AMFadeOutType)[self parseEnum:fadeOutType withDefaultValue:AM_FOT_NONE enumStrings:enumFadeOutType]];
		[chunk setFadeInDur:[self parseTime:fadeInDur]];
		[chunk setFadeOutDur:[self parseTime:fadeOutDur]];
		[chunk setActiveSound:[self parseInt:activeSound withDefaultValue:0]];
		
		// add the chunk to the last seen cue
		[lastCue addChunk:chunk];
		
    } else if ([elementName isEqualToString:@"sound"]) {
		
		AM_LOG_PARSER(@"Found Sound.");
		
		// retrieve sound attributes
		NSString *src = [attributeDict valueForKey:@"src"];
		NSString *pickPriority = [attributeDict valueForKey:@"pickPriority"];
		NSString *loopCount = [attributeDict valueForKey:@"loopCount"];
		NSString *setActive = [attributeDict valueForKey:@"setActive"];
		NSString *volume = [attributeDict valueForKey:@"volume"];
		NSString *pan = [attributeDict valueForKey:@"pan"];
		
		// create the sound if the source is valid
		if ([self checkPath:src]) {
			AM_LOG_PARSER(@"Sound source seems valid.");
			
			AMSound *sound = [[AMSound alloc] initWithParent:lastChunk parentCue:lastCue src:[basePath stringByAppendingString:src]];
			[sound autorelease];
			
			lastSound = sound;
			lastControllable = sound;
			
			// configure the sound
			[sound setPickPriority:[self parseInt:pickPriority withDefaultValue:1]];
			[sound setLoopCount:[self parseInt:loopCount withDefaultValue:1]];
			[self parseEventOrTimeList:setActive playable:sound action:AM_ACT_SET_ACTIVE];
			
			// configure mix controls
			if (volume != nil || pan != nil) {
				AMMix2DControl *mix2DControl = (AMMix2DControl *)[sound createControlWithName:AM_MIX_2D_CONTROL]; 
				mix2DControl.volume = [self parseInt:volume withDefaultValue:100];
				mix2DControl.pan = [self parseInt:pan withDefaultValue:0];
				AM_LOG_PARSER(@"Mix 2D parameters defined for the sound.");
			}
			
			// add the sound to the latest seen cue
			[lastChunk addSound:sound];
			AM_LOG_PARSER(@"Sound %d added to chunk", [[lastChunk getSounds] count] + 1);
			
		} else {
			AM_LOG_PARSER(@"Invalid sound source! Exceptions may arise.");
			[parser abortParsing];
		}
		
    } else if ([elementName isEqualToString:@"animate"]) {
		
		AM_LOG_PARSER(@"Found Animation.");
		
		// retrieve cue attributes
		NSString *idf = [attributeDict valueForKey:@"id"];
		NSString *attribute = [attributeDict valueForKey:@"attribute"];
		NSString *begin = [attributeDict valueForKey:@"begin"];
		NSString *dur = [attributeDict valueForKey:@"dur"];
		NSString *end = [attributeDict valueForKey:@"end"];
		NSString *quantize = [attributeDict valueForKey:@"quantize"];
		NSString *repeatCount = [attributeDict valueForKey:@"repeatCount"];
		NSString *repeatDur = [attributeDict valueForKey:@"repeatDur"];
		NSString *from = [attributeDict valueForKey:@"from"];
		NSString *to = [attributeDict valueForKey:@"to"];
		NSString *by = [attributeDict valueForKey:@"by"];
		NSString *values = [attributeDict valueForKey:@"values"];
		NSString *eventValue = [attributeDict valueForKey:@"eventValue"];
		NSString *fill = [attributeDict valueForKey:@"fill"];
		NSString *accumulate = [attributeDict valueForKey:@"accumulate"];
		
		// create the animation if the ID is valid
		if ([self checkID:idf]) {
			
			AM_LOG_PARSER(@"Animation ID '%@' is valid.", idf);
			
			AMAnimation *anim = [[AMAnimation alloc] initWithParent:lastControllable control:lastControl attribute:attribute];
			[anim autorelease];
			
			lastPlayable = anim;
			
			// register ID
			[AMSoundManager addKeyUID:[anim getUID] forID:idf];
			[AMSoundManager addKeyID:idf forObject:anim];
			
			// configure the animation
			[self parseEventOrTimeList:begin playable:anim action:AM_ACT_START];
			[anim setDur:[self parseTime:dur]];
			[self parseEventOrTimeList:end playable:anim action:AM_ACT_STOP];
			[anim setQuantize:[self parseTime:quantize]];
			[anim setRepeatCount:[self parseDouble:repeatCount withDefaultValue:1.0]];
			[anim setRepeatDur:[self parseTime:repeatDur]];
			[anim setFrom:[self parseValue:from]];
			[anim setTo:[self parseValue:to]];
			[anim setBy:[self parseValue:by]];
			[anim setValues:[self parseDoubleList:values]];
			[self parseEventOrTimeList:eventValue playable:anim action:AM_ACT_SET];
			[anim setFill:(AMFillType)[self parseEnum:fill withDefaultValue:AM_FIL_REMOVE enumStrings:enumFillType]];
			[anim setAccumulation:(AMAccumulationType)[self parseEnum:accumulate withDefaultValue:AM_ACC_NONE enumStrings:enumAccumulationType]];			
			
			// TODO: store animation in some array? SOundManager?
		}
		
    } else if ([elementName isEqualToString:@"tempoControl"]) {
		
		AM_LOG_PARSER(@"Found TempoControl.");
		
		AMTempoControl *tempoControl = nil;
		
		if (inMasterSection)
			tempoControl = (AMTempoControl *)[AMSoundManager createControlWithName:AM_TEMPO_CONTROL];
		else		
			tempoControl = (AMTempoControl *)[lastControllable createControlWithName:AM_TEMPO_CONTROL];
		
		if (tempoControl) {
			// configure control
			tempoControl.tempo = [self parseInt:[attributeDict valueForKey:@"tempo"] withDefaultValue:AM_DEFAULT_TEMPO];
			
			lastControl = AM_TEMPO_CONTROL;
		} else {
			lastControl = nil;
		}

    } else if ([elementName isEqualToString:@"triggerControl"]) {
		
		AM_LOG_PARSER(@"Found TriggerControl.");
		
		AMTriggerControl *triggerControl = nil;
		
		if (!inMasterSection)
			triggerControl = (AMTriggerControl *)[lastControllable createControlWithName:AM_TRIGGER_CONTROL];
		
		if (triggerControl) {
			// configure control
			triggerControl.chance = [self parseInt:[attributeDict valueForKey:@"chance"] withDefaultValue:100];
			triggerControl.autoStart  = [self parseTime:[attributeDict valueForKey:@"autoStart"]];
			
			lastControl = AM_TRIGGER_CONTROL;
		} else {
			lastControl = nil;
		}
		
    } else if ([elementName isEqualToString:@"spawnControl"]) {
		
		AM_LOG_PARSER(@"Found SpawnControl.");
		
		AMSpawnControl *spawnControl = nil;
		
		if (inMasterSection)
			spawnControl = (AMSpawnControl *)[AMSoundManager createControlWithName:AM_SPAWN_CONTROL];
		else		
			spawnControl = (AMSpawnControl *)[lastControllable createControlWithName:AM_SPAWN_CONTROL];
		
		if (spawnControl) {
			// configure control
			spawnControl.maxInstances = [self parseInt:[attributeDict valueForKey:@"maxInstances"] withDefaultValue:-1];
			
			lastControl = AM_SPAWN_CONTROL;
		} else {
			lastControl = nil;
		}
		
    } else if ([elementName isEqualToString:@"pitchControl"]) {
		
		AM_LOG_PARSER(@"Found PitchControl.");
		
		AMSoundControl *soundControl = nil;
		
		if (!inMasterSection)
			soundControl = (AMSoundControl *)[lastControllable createControlWithName:AM_SOUND_CONTROL];
		
		if (soundControl) {
			// configure control
			soundControl.pitch = [self parseInt:[attributeDict valueForKey:@"pitch"] withDefaultValue:0];
			
			lastControl = AM_SOUND_CONTROL;
		} else {
			lastControl = nil;
		}
		
    } else if ([elementName isEqualToString:@"rateControl"]) {
		
		AM_LOG_PARSER(@"Found RateControl.");
		
		AMSoundControl *soundControl = nil;
		
		if (!inMasterSection)
			soundControl = (AMSoundControl *)[lastControllable createControlWithName:AM_SOUND_CONTROL];
		
		if (soundControl) {
			// configure control
			soundControl.rate = [self parseInt:[attributeDict valueForKey:@"rate"] withDefaultValue:100000];
			
			lastControl = AM_SOUND_CONTROL;
		} else {
			lastControl = nil;
		}
		
    } else if ([elementName isEqualToString:@"volumeControl"]) {
		
		AM_LOG_PARSER(@"Found VolumeControl.");
		
		AMMix2DControl *mix2DControl = nil;
		
		if (inMasterSection)
			mix2DControl = (AMMix2DControl *)[AMSoundManager createControlWithName:AM_MIX_2D_CONTROL];
		else
			mix2DControl = (AMMix2DControl *)[lastControllable createControlWithName:AM_MIX_2D_CONTROL];
		
		if (mix2DControl) {
			// configure control
			mix2DControl.volume = [self parseInt:[attributeDict valueForKey:@"level"] withDefaultValue:100];
			mix2DControl.mute = [self parseBoolean:[attributeDict valueForKey:@"mute"] withDefaultValue:NO];
			
			lastControl = AM_MIX_2D_CONTROL;
		} else {
			lastControl = nil;
		}
		
    } else if ([elementName isEqualToString:@"panControl"]) {
		
		AM_LOG_PARSER(@"Found PanControl.");
		
		AMMix2DControl *mix2DControl = nil;
		
		if (inMasterSection)
			mix2DControl = (AMMix2DControl *)[AMSoundManager createControlWithName:AM_MIX_2D_CONTROL];
		else
			mix2DControl = (AMMix2DControl *)[lastControllable createControlWithName:AM_MIX_2D_CONTROL];
		
		if (mix2DControl) {
			// configure control
			mix2DControl.pan = [self parseInt:[attributeDict valueForKey:@"pan"] withDefaultValue:0];
			
			lastControl = AM_MIX_2D_CONTROL;
		} else {
			lastControl = nil;
		}
		
    }
		
		
		
		
		
	
	/*
	 
	 // An entry in the RSS feed represents an earthquake, so create an instance of it.
	 //        self.currentEarthquakeObject = [[Earthquake alloc] init];
	 // Add the new Earthquake object to the application's array of earthquakes.
	 //        [(id)[[UIApplication sharedApplication] delegate] performSelectorOnMainThread:@selector(addToEarthquakeList:) withObject:self.currentEarthquakeObject waitUntilDone:YES];
	 //        return;
	 //    }
	 
	 if ([elementName isEqualToString:@"link"]) {
	 NSString *relAtt = [attributeDict valueForKey:@"rel"];
	 if ([relAtt isEqualToString:@"alternate"]) {
	 NSString *link = [attributeDict valueForKey:@"href"];
	 link = [NSString stringWithFormat:@"http://earthquake.usgs.gov/%@", link];
	 
	 }
	 } else if ([elementName isEqualToString:@"title"]) {
	 // Create a mutable string to hold the contents of the 'title' element.
	 // The contents are collected in parser:foundCharacters:.
	 
	 }
	 */
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{     
    if (qName) {
        elementName = qName;
    }
    
	if ([elementName isEqualToString:@"masterSection"]) {
		inMasterSection = NO;
	}
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
	/*
	 if (self.contentOfCurrentEarthquakeProperty) {
	 // If the current element is one whose content we care about, append 'string'
	 // to the property that holds the content of the current element.
	 [self.contentOfCurrentEarthquakeProperty appendString:string];
	 }*/
}


// Private functions

- (BOOL)checkID:(NSString *)_id
{
	return [AMSoundManager checkID:_id];
}

- (BOOL)checkPath:(NSString *)_path
{
	if (_path != nil)
		return [[NSFileManager defaultManager] fileExistsAtPath:[basePath stringByAppendingString:_path] isDirectory:NULL];
	return false;
}

- (void)parseEventOrTimeList:(NSString *)_str playable:(id<AMPlayable>)_playable action:(AMActionType)_action
{
	AM_LOG_PARSER(@"Parsing Time/Event list...");
	
	if (_str != nil && _playable != nil) {
		NSArray *listElements = [self parseList:_str];
		
		if (listElements != nil) {
			// parse each element
			for (NSString *element in listElements) {
				
				AM_LOG_PARSER(@"Found list element: '%@'", element);
				
				// trim unwanted characters
				NSString *elem = [element stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
				
				// check the type of element
				if ([elem hasPrefix:@"rand("] || isdigit([elem characterAtIndex:0])) {
					// time value
					AMTime *time = [self parseTime:elem];
					
					if (time != nil) {
						// register the action
						AMScheduler *scheduler = [AMSoundManager getScheduler];
						[scheduler registerTimeAction:time playable:_playable action:_action permanent:YES];
					}
					
				} else {
					// event value
					AMTime *delay = nil;
					NSString *event = [self parseEvent:elem delay:&delay];
					
					if (event != nil) {
						// register the action
						AMScheduler *scheduler = [AMSoundManager getScheduler];
						[scheduler registerEventAction:event playable:_playable action:_action delay:delay permanent:YES];					
					}
				}
			}
		}
	}
	
	AM_LOG_PARSER(@"Parsing Time/Event done.");
}

- (AMValue *)parseValue:(NSString *)_str
{
	AMValue *value = nil;
	
	if (_str != nil) {
		// Parse the value
		NSCharacterSet *separators = [NSCharacterSet characterSetWithCharactersInString:@"(,)"];
		NSScanner *scanner = [NSScanner scannerWithString:_str];
		[scanner setCaseSensitive:YES];
		double val1, val2, val3;
		
		// parse a random expression
		if([scanner scanString:@"rand(" intoString:NULL]) {
			
			// parse the first two numbers
			if ([scanner scanDouble:&val1] && [scanner scanCharactersFromSet:separators intoString:NULL]
				&& [scanner scanDouble:&val2]) {
				
				// check if there's a 3rd number
				if ([scanner scanCharactersFromSet:separators intoString:NULL] || [scanner scanDouble:&val3]) {
					value = [[AMRandomValue alloc] initWithMinValue:val1 maxValue:val2 interval:val3];
					[value autorelease];
					
					AM_LOG_PARSER(@"Found random exp.: min=%f, max=%f, int=%f", val1, val2, val3);
					
				} else {
					value = [[AMRandomValue alloc] initWithMinValue:val1 maxValue:val2];
					[value autorelease];
					
					AM_LOG_PARSER(@"Found random exp.: min=%f, max=%f", val1, val2);
				}
				
				// skip the last ')'
				[scanner scanCharactersFromSet:separators intoString:NULL];
			}
		} else {
			// parse a single value
			if ([scanner scanDouble:&val1]) {
				value = [[AMValue alloc] initWithValue:val1];
				[value autorelease];
				
				AM_LOG_PARSER(@"Found simple value: %f", val1);
			}
		}		
	}
	
	return value;
}

- (AMTime *)parseTime:(NSString *)_str
{
	AMTime *time = nil;
	
	if (_str != nil) {
		// Parse the time value
		AMValue *value = nil;	
		NSCharacterSet *separators = [NSCharacterSet characterSetWithCharactersInString:@"(,)"];
		NSScanner *scanner = [NSScanner scannerWithString:_str];
		[scanner setCaseSensitive:YES];
		double val1, val2, val3, factor = 1.0;
		BOOL isTempoBased = false;
		
		// parse a random expression
		if([scanner scanString:@"rand(" intoString:NULL]) {
			
			// parse the first two numbers
			if ([scanner scanDouble:&val1] && [scanner scanCharactersFromSet:separators intoString:NULL]
				&& [scanner scanDouble:&val2]) {
				
				// check if there's a 3rd number
				if ([scanner scanCharactersFromSet:separators intoString:NULL] || [scanner scanDouble:&val3]) {
					value = [[AMRandomValue alloc] initWithMinValue:val1 maxValue:val2 interval:val3];
					[value autorelease];
					
					AM_LOG_PARSER(@"Found random exp.: min=%f, max=%f, int=%f", val1, val2, val3);
					
				} else {
					value = [[AMRandomValue alloc] initWithMinValue:val1 maxValue:val2];
					[value autorelease];
					
					AM_LOG_PARSER(@"Found random exp.: min=%f, max=%f", val1, val2);
				}
				
				// skip the last ')'
				[scanner scanCharactersFromSet:separators intoString:NULL];
			}
		} else {
			// parse a single value
			if ([scanner scanDouble:&val1]) {
				value = [[AMValue alloc] initWithValue:val1];
				[value autorelease];
				
				AM_LOG_PARSER(@"Found simple value: %f", val1);
			}
		}
		
		// parse unit if present
		if ([scanner scanString:@"ms" intoString:NULL]) {
			factor = 0.001;
		} else if ([scanner scanString:@"m" intoString:NULL]) {
			factor = 60;
		} else if ([scanner scanString:@"B" intoString:NULL]) {
			isTempoBased = YES;
			factor = AM_TEMPO_RESOLUTION * 4;	// only 4/4 time signature supported at the moment.
		} else if ([scanner scanString:@"b" intoString:NULL]) {
			isTempoBased = YES;
			factor = AM_TEMPO_RESOLUTION;
		} else if ([scanner scanString:@"t" intoString:NULL]) {
			isTempoBased = YES;
		}
		
		AM_LOG_PARSER(@"Unit factor: %f, time based: %d", factor, (int)isTempoBased);
		
		// create the AMTime object
		if (value != nil) {
			// apply correction factor if needed
			if (factor != 1.0) {
				if ([value class] == [AMRandomValue class]) {
					AMRandomValue *randVal = (AMRandomValue *)value;
					[randVal setMinValue:[randVal minValue] * factor];
					[randVal setMaxValue:[randVal maxValue] * factor];
					[randVal setInterval:[randVal interval] * factor];
				} else {
					[value setValue:[value value] * factor];
				}
			}
			
			time = [[AMTime alloc] initWithTime:value isTimeTempoBased:isTempoBased];
			
			AM_LOG_PARSER(@"Created time value.");
		}	
	}
	
	return time;
}

- (NSString *)parseEvent:(NSString *)_str delay:(AMTime **)_delay
{
	NSString *event = nil;
	
	if (_str != nil) {
		NSCharacterSet *separators = [NSCharacterSet characterSetWithCharactersInString:@"+-"];
		NSScanner *scanner = [NSScanner scannerWithString:_str];
		NSString *curr;
		
		// scan the event name
		if ([scanner scanUpToCharactersFromSet:separators intoString:&curr]) {
			// get the event
			event = [curr stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
			
			AM_LOG_PARSER(@"Found event named '%@'", event);
			
			// check if there's an offset delay
			if ([scanner scanString:@"+" intoString:NULL]) {
				
				AM_LOG_PARSER(@"Found positive offset delay.");
				*_delay = [self parseTime:[[scanner string]substringFromIndex:[scanner scanLocation]]];
				
			} else if ([scanner scanString:@"-" intoString:NULL]) {
				
				AM_LOG_PARSER(@"Found negative offset delay.");
				*_delay = [self parseTime:[[scanner string]substringFromIndex:[scanner scanLocation]]];
				AMValue *val = [*_delay time];
				
				// checks the type of delay and adjust the offset
				if ([val class] == [AMRandomValue class]) {
					AMRandomValue *randVal = (AMRandomValue *)val;
					[randVal setMinValue:-[randVal maxValue]];
					[randVal setMaxValue:-[randVal minValue]];
					
				} else {
					val.value = -val.value;
				}
				
			} else {
				*_delay = nil;
				
				AM_LOG_PARSER(@"No delay for the event.");
			}
		}	
	}
	
	return event;
}

- (NSArray *)parseList:(NSString *)_str
{
	NSArray *elements = nil;
	NSMutableArray *listElements = [NSMutableArray arrayWithCapacity:5];
	NSCharacterSet *connectors = [NSCharacterSet characterSetWithCharactersInString:@"+-,"];
	BOOL appendNext = YES;
	int numElem = 0;	// number of elements seen without a connector
	
	if (_str != nil) {
		// split the string into elements
		NSString *list = [_str stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		elements = [list componentsSeparatedByString:LIST_SEPARATOR];
		
		// combine the elements when needed
		if (elements != nil) {
			NSMutableString *elem = nil;
			
			// parse each element
			for (NSString *currStr in elements) {
				
				// change append status only if there is significant string data
				if (currStr != nil && [currStr length] > 0) {
					if (![connectors characterIsMember:[currStr characterAtIndex:[currStr length]-1]]) {
						appendNext = NO;
						numElem++;
					} else {
						appendNext = YES;
						
						// reset number of elements when found a connector is found
						numElem = 0;
					}
				} else {
					// continue to append until a non-null string is found
					appendNext = YES;
				}
				
				// append current string if needed
				if (elem == nil) {
					elem = [NSMutableString stringWithString:currStr];
					numElem = 1;
				} else {
					if (appendNext || numElem < 2) {
						// continue previous element
						[elem appendString:currStr];
					} else {
						// store element
						[listElements addObject:elem];						
						
						elem = [NSMutableString stringWithString:currStr];
						numElem = 1;
					}				
				}
				
				//				AM_LOG_PARSER(@"**CURR STR: '%@', APP:%d, NUM:%d", currStr, (int)appendNext, numElem);
			}
			
			// store last element if needed
			if (elem != nil) {
				[listElements addObject:elem];
			}
		}
	}
	
	if ([listElements count] > 0)
		return listElements;
	else
		return nil;
}

- (NSArray *)parseDoubleList:(NSString *)_str
{
	NSArray *elements = nil;
	NSMutableArray *doubleList = nil;
	double val;
	
	AM_LOG_PARSER(@"Parsing list of values...");
	
	if (_str != nil) {
		doubleList = [[NSMutableArray alloc] initWithCapacity:5];
		[doubleList autorelease];
		NSString *list = [_str stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		elements = [list componentsSeparatedByString:LIST_SEPARATOR];
		
		if (elements != nil) {
			// parse each element
			for (NSString *currStr in elements) {
				// get value only if there is significant string data
				if (currStr != nil && [currStr length] > 0) {
					// get value
					AM_LOG_PARSER(@"Potential value: '%@'", currStr);
					NSScanner *scanner = [NSScanner scannerWithString:currStr];
					if ([scanner scanDouble:&val]) {
						[doubleList addObject:[NSNumber numberWithDouble:val]];
						AM_LOG_PARSER(@"Value is valid : '%f'", val);
					}
				}
			}
		}
	}
	
#ifdef AM_DEBUG_PARSER
	if (doubleList != nil) {
		AM_LOG_PARSER(@"Total values found: %d", [doubleList count]);
	}
#endif
	
	AM_LOG_PARSER(@"Parsing list of values done.");
	
	return doubleList;
}
	
- (NSArray *)parseReferenceList:(NSString *)_str
{
	NSArray *elements = nil;
	NSMutableArray *refList = nil;
	
	AM_LOG_PARSER(@"Parsing cue references list...");
	
	if (_str != nil) {
		refList = [[NSMutableArray alloc] initWithCapacity:5];
		[refList autorelease];
		NSString *list = [_str stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		elements = [list componentsSeparatedByString:LIST_SEPARATOR];
		
		if (elements != nil) {
			// parse each element
			for (NSString *currStr in elements) {
				// resolve reference only if there is significant string data
				if (currStr != nil && [currStr length] > 0) {
					// resolve reference
					NSObject *obj = [AMSoundManager getObjectFromID:currStr];
					AM_LOG_PARSER(@"Potential cue reference: '%@'", currStr);
					
					// if a valid referenced object is found, then add it to the list
					if (obj != nil) {
						[refList addObject:obj];
						AM_LOG_PARSER(@"Cue reference is valid : '%@'", currStr);
					}
				}
			}
		}
	}
	
#ifdef AM_DEBUG_PARSER
	if (refList != nil) {
		AM_LOG_PARSER(@"Total valid cue references found: %d", [refList count]);
	}
#endif
	
	AM_LOG_PARSER(@"Parsing cue references done.");
	
	return refList;
}

- (int)parseInt:(NSString *)_str withDefaultValue:(int)_defaultVal
{
	if (_str != nil) {
		int val;
		NSScanner *scanner = [NSScanner scannerWithString:_str];
		
		if ([scanner scanInt:&val])
			return val;		
	}
	return _defaultVal;
}
			 
- (int)parseDouble:(NSString *)_str withDefaultValue:(double)_defaultVal
{
	if (_str != nil) {
		double val;
		NSScanner *scanner = [NSScanner scannerWithString:_str];
		
		if ([scanner scanDouble:&val])
			return val;		
	}
	return _defaultVal;
}

- (BOOL)parseBoolean:(NSString *)_str withDefaultValue:(BOOL)_defaultVal
{
	if (_str != nil) {
		NSScanner *scanner = [NSScanner scannerWithString:_str];
		
		if ([scanner scanString:@"true" intoString:NULL])
			return YES;
		else if ([scanner scanString:@"false" intoString:NULL])
			return NO;
	}
	return _defaultVal;
}

- (int)parseEnum:(NSString *)_str withDefaultValue:(int)_defaultVal enumStrings:(NSArray *)_enumStrings
{
	if (_str != nil && _enumStrings != nil) {
		NSScanner *scanner = [NSScanner scannerWithString:_str];
		int num = 0;
		
		for (NSString *elem in _enumStrings) {
			if ([scanner scanString:elem intoString:NULL])
				break;
			num++;
		}
		
		if (num < [_enumStrings count])
			return num;
	}
	return _defaultVal;
}


@end
