/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMScheduler.h
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMTimeAction.h"
#import "AMEventAction.h"
#import "AMAnimation.h"
#import "AMTime.h"
#import "AMPlayable.h"
#import "a2ml.h"
#import <Foundation/Foundation.h>


/**
 An A2ML scheduler implementation.
 */
@interface AMScheduler : NSObject {
	// Scheduler attributes
	double currentTime;
	BOOL paused;
	
	// Internal variables
	NSDate *initialTime;				/* Initial time at which the scheduler started. */
	NSNumber *previousTimeIndex;		/* The previous time index used. */
	
	NSMutableDictionary *animations;		/* Managed animations. */
	NSMutableSet *eventActions;				/* The events/actions matching table. */
	NSMutableDictionary *timeActions;		/* The times/actions matching table. */
	
	NSMutableSet *startedAnimations;		/* The list of started animations. */
	NSMutableDictionary *managedObjects;	/* Cue related objects managed by the scheduler. */
}

@property (nonatomic, readonly) double currentTime;	/**< The Internal current time of the scheduler, in seconds. */
@property (nonatomic, readonly) BOOL paused;		/**< Is the scheduler running or paused? */

/**
 * Registers a permanent or single-use scheduler action and link it to a particular event.
 * If the action is not set as permanent, the scheduler will dismiss this action after
 * the action has been performed.
 * @param event The event that will trigger the action.
 * @param playable The playable object on which the action will apply.
 * @param action The action type, as defined by the AMActionType enumeration.
 * @param delay The delay after which the action will be performed once the event has been triggered.
 * @param permanent Use YES to set the action as permanent or NO to set it as a single-use action.
 * @return The new AMEventAction object created or nil if none was created.
 * @see AMEventAction 
 */
- (AMEventAction *)registerEventAction:(NSString *)_event playable:(id)_playable action:(AMActionType)_action delay:(AMTime *)_delay permanent:(BOOL)_permanent;

/**
 * Registers a permanent or single-use scheduler action and link it to a particular time.
 * If the action is not set as permanent, the scheduler will dismiss this action after
 * the action has been performed.
 * @param time The time on which the action will be triggered.
 * @param playable The playable object on which the action will apply.
 * @param action The action type, as defined by the AMActionType enumeration.
 * @param permanent Use YES to set the action as permanent or NO to set it as a single-use action.
 * @return The new AMTimeAction object created or nil if none was created.
 * @see AMTimeAction
 */
- (AMTimeAction *)registerTimeAction:(AMTime *)_time playable:(id)_playable action:(AMActionType)_action permanent:(BOOL)_permanent;

/**
 Removes the specified AMEventAction object from the scheduler.
 @param eventAction The AMEventAction object to remove.
 */
- (void)removeEventAction:(AMEventAction *)_eventAction;

/**
 Increments the scheduler time and performs all time-related scheduler actions.
 The value of the increment will be automatically calculated, based on the last time this method was called.
 The initial time is initialized on the first call of this method, or on the first call after a time reset was requested.
 This method should be invoked on a regular time interval to allow the scheduler to work properly (usually done by the sound manager).
 */
- (void)incrementTime;

/**
 Starts the scheduler.
 */
- (void)start;

/**
 Pauses the scheduler.
 */
- (void)pause;

/**
 Resets the current time of the scheduler, and pause it.
 Be careful as resetting the current scheduler time may introduce side effects on the behavior of scheduled objets like animations or fades,
 if they are currently running.
 */
- (void)resetTime;

/**
 Completely resets the scheduler, including removing all time and event actions.
 The scheduler is stopped before the cleanup.
 */
- (void)resetAll;

@end
