/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMScheduler.mm
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMScheduler.h"
#import "AMTempoControl.h"
#import "AMControllableHierarchy.h"
#import "AMSoundManager.h"	// need to be there to avoid cross-referencing problems.


@implementation AMScheduler

@synthesize currentTime;
@synthesize paused;

- (id)init
{
	if (self = [super init]) {
		initialTime = nil;
		currentTime = 0;
		paused = YES;
		previousTimeIndex = nil;
		animations = [[NSMutableDictionary alloc] init];
		eventActions = [[NSMutableSet alloc] init];
		timeActions = [[NSMutableDictionary alloc] init];
		startedAnimations = [[NSMutableSet alloc] init];
		managedObjects = [[NSMutableDictionary alloc] init];
	}
	return self;
}

- (AMEventAction *)registerEventAction:(NSString *)_event playable:(id)_playable action:(AMActionType)_action delay:(AMTime *)_delay permanent:(BOOL)_permanent
{
	AMEventAction *eventAction = nil;
	
	eventAction = [[AMEventAction alloc] initWithPlayable:_playable event:_event action:_action delay:_delay permanent:_permanent];
	[eventActions addObject:eventAction];
	
	return [eventAction autorelease];
}

- (AMTimeAction *)registerTimeAction:(AMTime *)_time playable:(id)_playable action:(AMActionType)_action permanent:(BOOL)_permanent
{
	AMTimeAction *timeAction = nil;
	int tempo = [(AMTempoControl *)[(id<AMControllableHierarchy>)_playable getClosestControlWithName:AM_TEMPO_CONTROL] tempo];
	
	if ((_action == AM_ACT_START || _action == AM_ACT_STOP) && [_playable respondsToSelector:@selector(quantize)]) {
		AMTime *quantizer = [_playable quantize];
		timeAction = [[AMTimeAction alloc] initWithPlayable:_playable begin:[_time calcTimeWithTempo:tempo quantizer:[quantizer calcTimeWithTempo:tempo] isQuantizerTempoBased:[quantizer isTimeTempoBased]] action:_action permanent:_permanent];
	} else {
		timeAction = [[AMTimeAction alloc] initWithPlayable:_playable begin:[_time calcTimeWithTempo:tempo] action:_action permanent:_permanent];
	}

	NSNumber *timeIndex = [NSNumber numberWithDouble:(roundf([timeAction getBeginTime]))];
	
	// get the group of actions for that time index
	NSMutableSet *actions = [timeActions objectForKey:timeIndex];
	
	// create a new group of actions if it doesn't exist
	if (actions == nil) {
		actions = [NSMutableSet setWithCapacity:3];
		[timeActions setObject:actions forKey:timeIndex];
	}
	
	[actions addObject:timeAction];
	
	return [timeAction autorelease];
}

- (void)removeEventAction:(AMEventAction *)_eventAction
{
	[eventActions removeObject:_eventAction];
}

- (void)incrementTime
{	
	double timeInterval = 0;	// time interval since the last method call
	
	if (initialTime == nil) {
		// initialize the time origin
		initialTime = [[NSDate alloc] init];
		currentTime = 0;
		timeInterval = 0.0001;		// needed to trigger 0s timed actions
		previousTimeIndex = [[NSNumber alloc] initWithDouble:0];
	} else {
		// increment time
		double previousTime = currentTime;
		currentTime = -(double)[initialTime timeIntervalSinceNow];
		timeInterval = currentTime - previousTime;		
	}
	
	// manage time-based actions
	NSNumber *timeIndex = [NSNumber numberWithDouble:(roundf(currentTime))];
	NSMutableSet *actions = [timeActions objectForKey:timeIndex];
	
#ifdef AM_LOG_SCHEDULER
	//AM_LOG_SCHEDULER(@"Time: %f, Interval: %f", currentTime, timeInterval);
	if (![previousTimeIndex isEqualToNumber:timeIndex]) {
		AM_LOG_SCHEDULER(@"Current Time = %.2fs", currentTime);
	}
#endif
	
	// do at most 2 iterations, 1 with current time index and 1 with the previous one if needed
	NSNumber *oldTimeIndex = nil;
	while(true) {
		if (actions != nil) {
			for (AMTimeAction *timeAction in actions) {
				// remove single-use actions after execution
				if ([timeAction doAction:currentTime timeInterval:timeInterval]) {
					[actions removeObject:timeAction];
				}
			}

		}
		
		// remove the set of actions if it's empty
		if ([actions count] == 0) {
			if (oldTimeIndex == nil) {
				[timeActions removeObjectForKey:timeIndex];
			} else {
				[timeActions removeObjectForKey:oldTimeIndex];
			}
		}
		
		// check if another iteration is needed
		if ([previousTimeIndex isEqualToNumber:timeIndex]) {
			// no more iteration needed, exit loop
			break;
		}
		
		actions = [timeActions objectForKey:previousTimeIndex];
		oldTimeIndex = previousTimeIndex;
		previousTimeIndex = [timeIndex retain];
	}
	
	[oldTimeIndex release];
	
	// manage animations
	for (AMAnimationInstance *animation in animations) {
		[animation animate:currentTime];
	}
}

- (void)start
{
	if (initialTime != nil) {
		double now = (double)[initialTime timeIntervalSinceNow];
		double offset = now - currentTime;
		[initialTime addTimeInterval:offset];
	}
	
	paused = NO;
	AM_LOG_SCHEDULER(@"Started.");
}

- (void)pause
{
	paused = YES;
	[self incrementTime];
	AM_LOG_SCHEDULER(@"Paused.");
}

- (void)resetTime
{
	paused = YES;
	[initialTime release];
	initialTime = nil;
	currentTime = 0;
	AM_LOG_SCHEDULER(@"Time reset.");
}

- (void)resetAll
{
	paused = YES;
	[initialTime release];
	initialTime = nil;
	currentTime = 0;
	
	[previousTimeIndex release];
	previousTimeIndex = nil;
	
	[animations removeAllObjects];
	[eventActions removeAllObjects];
	[timeActions removeAllObjects];
	[startedAnimations removeAllObjects];
	[managedObjects removeAllObjects];
	
	AM_LOG_SCHEDULER(@"Full reset.");
}

-(void)dealloc 
{
	// release vars
	[initialTime release];
	[previousTimeIndex release];
	
	[animations release];
	[eventActions release];
	[timeActions release];
	[startedAnimations release];
	[managedObjects release];
	
	[super dealloc];
}

@end
