/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  DocumentDetailTableViewController.mm
//  aria
//
//  Created by Yohan on 08/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "DocumentDetailTableViewController.h"


// Constants
#define TF_NAME_TAG		0		// tag of the name textfield
#define TF_URL_TAG		1		// tag of the URL textfield


@implementation DocumentDetailTableViewController

@synthesize document;
@synthesize modalMode;


#pragma mark -
#pragma mark Initialization

- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    if ((self = [super initWithStyle:style])) {
		modalMode = NO;
		document = nil;
    }
    return self;
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return YES;
}

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];

    // init view
	if (!modalMode) {
		if (document.online)
			self.navigationItem.rightBarButtonItem = self.editButtonItem;		

	} else {
		// create navigation bar buttons
		UIBarButtonItem *doneButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSave target:self action:@selector(saveDocument)];
		self.navigationItem.rightBarButtonItem = doneButton;
		[doneButton release];

		UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(cancelNewDocument)];
		self.navigationItem.leftBarButtonItem = cancelButton;
		[cancelButton release];
		
		self.navigationItem.title = @"Add document";
		self.editing = YES;
	}
	
	// forbids user selection
	self.tableView.allowsSelection = NO;

	// init vars
	AppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
	docTableViewController = [[appDelegate rootViewController] documentTableViewController];
	
	// configure footer button if needed
	footerView.frame = CGRectMake(0.0, 0.0, self.tableView.bounds.size.width, footerView.frame.size.height);
	footerView.backgroundColor = [UIColor clearColor];
	self.tableView.tableFooterView = footerView;
	
	if (document)
		[footerView setHidden:!document.online];
	
	if (modalMode)
		[footerView setHidden:YES];
	
}

- (void)viewWillDisappear:(BOOL)animated {
	// refresh online documents
	if (!modalMode)
		[docTableViewController.tableView reloadSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationNone];
	
    [super viewWillDisappear:animated];
}

- (void)setDocument:(Document *)_document {
	[document release];
	document = [_document retain];
	
	[footerView setHidden:!document.online];
}

#pragma mark -
#pragma mark Table view data source

- (BOOL)checksAndSaveDocument
{
	BOOL isDocOK = NO;
	
	NSString *candidateName = (nameTextField.text ? [nameTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] : @"");
	NSString *candidateUrl = (urlTextField.text ? [urlTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] : @"");	
	document.description = (descTextView.text ? [descTextView.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] : @"");
	
	if (![candidateName isEqualToString:@""] && ![candidateUrl isEqualToString:@""]) {
		
		// check if document is unique
		BOOL unique = YES;
		for (Document *doc in docTableViewController.onlineDocumentsList)
			if ([candidateName isEqualToString:doc.name] && (doc != document))
				unique = NO;
		
		if (unique) {
			// check if URL is valid
			if ([DownloadManager isValidUrl:candidateUrl]) {

				// save old path & values
				NSString *path = (NSString *)[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
				NSString *oldPath = [path stringByAppendingPathComponent:document.name];
				NSString *oldName = [document.name retain];

				// save propeties
				document.name = candidateName;
				document.url = candidateUrl;
				isDocOK = YES;
				
				// manage folders
				path = [path stringByAppendingPathComponent:document.name];
				NSFileManager *fileManager = [NSFileManager defaultManager];
				
				if (![fileManager fileExistsAtPath:path]) {
					if (oldName && ![oldName isEqualToString:@""] && [fileManager fileExistsAtPath:oldPath]) {
						// move folder and data
						[fileManager moveItemAtPath:oldPath toPath:path error:NULL];
					} else {
						// create folder
						[fileManager createDirectoryAtPath:path withIntermediateDirectories:YES attributes:nil error:NULL];
					}
				}
				
				[oldName release];
				
			} else {
				UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:@"Error" message:@"The specified URL is invalid or its protocol is unsupported." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
				[alert show];
			}
		} else {
			UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:@"Error" message:@"The name of the document must be unique." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
			[alert show];
		}
	} else {
		UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:@"Missing info" message:@"You need to provide at least a name and a valid URL." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
		[alert show];
	}	
	
	return isDocOK;
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
	// end of edition
	if (!editing) {
		// check if document is valid
		if (![self checksAndSaveDocument])
			return;
	}
	
	[super setEditing:editing animated:animated];

	// need to be at both for fade animation
	footerView.alpha = editing ? 0 : 1;
	
	if (!editing) {
		// force end of edition
		if (nameTextField.editing)
			[nameTextField resignFirstResponder];
		if (urlTextField.editing) 
			[urlTextField resignFirstResponder];
		if ([descTextView isFirstResponder]) 
			[descTextView resignFirstResponder];
	}
	
	// change cells
	[self.tableView reloadSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:(modalMode?UITableViewRowAnimationNone:(editing?UITableViewRowAnimationLeft:UITableViewRowAnimationRight))];
	[self.tableView setNeedsDisplay];
	
	if (editing)
		[nameTextField becomeFirstResponder];

	// need to be at both for fade animation
	footerView.alpha = editing ? 0 : 1;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return 3;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
	if (indexPath.row == 2) {
		if (self.editing) {
			return 112;
		} else {
			UITableViewCell *cell = [self tableView:tableView cellForRowAtIndexPath:indexPath];
			
			// TODO: more elegant solution?
			NSString *text = cell.detailTextLabel.text; 
			CGFloat height = [text sizeWithFont:[UIFont systemFontOfSize:17] constrainedToSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 96, 9999) lineBreakMode:UILineBreakModeWordWrap].height;
			return MAX(44, height); 			
		}
	}
	return 44;
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    UITableViewCell *cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue2 reuseIdentifier:nil] autorelease];
    
    // configure the cell
	switch (indexPath.row) {
		case 0:		// name
		{
			if (self.editing) {
				nameTextField = [[UITextField alloc] initWithFrame:CGRectMake(93, 10, 210, 22)];
				nameTextField.adjustsFontSizeToFitWidth = YES;
				nameTextField.placeholder = @"new document";
				nameTextField.keyboardType = UIKeyboardTypeDefault;
				nameTextField.returnKeyType = UIReturnKeyNext;
				nameTextField.autocorrectionType = UITextAutocorrectionTypeNo;
				nameTextField.autocapitalizationType = UITextAutocapitalizationTypeNone;
				nameTextField.textAlignment = UITextAlignmentLeft;			
				nameTextField.clearButtonMode = UITextFieldViewModeNever;
				nameTextField.delegate = self;
				nameTextField.tag = TF_NAME_TAG;
				nameTextField.text = document.name;
				[cell addSubview:nameTextField];
				[nameTextField release];
			} else {
				cell.detailTextLabel.text = document.name;
			}
			
			cell.textLabel.text = @"Name";			
			break;
		}
		case 1:		// url
		{
			if (self.editing) {
				urlTextField = [[UITextField alloc] initWithFrame:CGRectMake(93, 10, 210, 22)];
				urlTextField.adjustsFontSizeToFitWidth = YES;
				urlTextField.placeholder = @"http://";
				urlTextField.keyboardType = UIKeyboardTypeURL;
				urlTextField.returnKeyType = UIReturnKeyNext;
				urlTextField.autocorrectionType = UITextAutocorrectionTypeNo;
				urlTextField.autocapitalizationType = UITextAutocapitalizationTypeNone;
				urlTextField.textAlignment = UITextAlignmentLeft;			
				urlTextField.clearButtonMode = UITextFieldViewModeNever;
				urlTextField.delegate = self;
				urlTextField.tag = TF_URL_TAG;
				urlTextField.text = document.url;
				
				[cell addSubview:urlTextField];
				[urlTextField release];
			} else {
				cell.detailTextLabel.text = document.url;
			}
			
			cell.textLabel.text = @"URL";
			break;
		}
		case 2:		// description
			if (self.editing) {
				descTextView = [[UITextView alloc] initWithFrame:CGRectMake(85, 4, 220, 102)];
				descTextView.font = urlTextField.font;
				descTextView.keyboardType = UIKeyboardTypeDefault;
				descTextView.autocorrectionType = UITextAutocorrectionTypeNo;
				descTextView.autocapitalizationType = UITextAutocapitalizationTypeNone;
				descTextView.textAlignment = UITextAlignmentLeft;
				descTextView.editable = YES;
				descTextView.contentInset = UIEdgeInsetsMake(-10,0,0,0);	// offset it up for better look
				descTextView.text = document.description;
				
				[cell addSubview:descTextView];
				[descTextView release];
				cell.detailTextLabel.text = @" ";
			} else {
				cell.detailTextLabel.text = document.description;
				cell.detailTextLabel.numberOfLines = 0;
				cell.detailTextLabel.lineBreakMode = UILineBreakModeWordWrap;
			}			
			
			cell.textLabel.text = @"Description";
			break;
		default:
			break;
	}
    
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
	return NO;
}

#pragma mark -
#pragma mark View modes and callbacks

- (void)saveDocument
{
	if ([self checksAndSaveDocument]) {		
		// add document and update document list
		[docTableViewController.onlineDocumentsList addObject:document];
		[docTableViewController.tableView reloadSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationFade];
		
		[self dismissModalViewControllerAnimated:YES];
	}
}

- (void)cancelNewDocument
{	
	[self dismissModalViewControllerAnimated:YES];
}

- (IBAction)reloadOnlineDocument
{
	UIAlertView *alert = [[[UIAlertView alloc] init] autorelease];
	[alert setTitle:@"Reload from URL"];
	[alert setMessage:@"What do you want to reload?"];
	[alert setDelegate:self];
	[alert addButtonWithTitle:@"A2ML + Sounds"];
	[alert addButtonWithTitle:@"A2ML Only"];
	[alert addButtonWithTitle:@"Cancel"];
	[alert show];	
}

#pragma mark -
#pragma mark Alert view delegate

- (void)alertView:(UIAlertView *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
	switch (buttonIndex) {
		case 0:		// A2ML + sounds
		{
			DownloadViewController *downloadViewController = [[DownloadViewController alloc] initWithNibName:@"DownloadViewController" bundle:nil];
			downloadViewController.document = document;
			downloadViewController.reloadSounds = YES;
			[downloadViewController setModalTransitionStyle:UIModalTransitionStyleCrossDissolve];
			[[OverlayViewController sharedOverlayViewController] presentModalViewController:downloadViewController animated:YES];
			[downloadViewController release];
			break;
		}
		case 1:		// A2ML only
		{
			DownloadViewController *downloadViewController = [[DownloadViewController alloc] initWithNibName:@"DownloadViewController" bundle:nil];
			downloadViewController.document = document;
			downloadViewController.reloadSounds = NO;
			[downloadViewController setModalTransitionStyle:UIModalTransitionStyleCrossDissolve];
			[[OverlayViewController sharedOverlayViewController] presentModalViewController:downloadViewController animated:YES];
			[downloadViewController release];
			break;
		}
		default:	// cancel
			return;
			break;
	}
}

#pragma mark -
#pragma mark Text field delegate

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
	switch (textField.tag) {
		case TF_NAME_TAG:
			[urlTextField becomeFirstResponder];
			break;
		case TF_URL_TAG:
			[descTextView becomeFirstResponder];
			return NO;
			break;
		default:
			break;
	}
	return YES;
}

#pragma mark -
#pragma mark Memory management

- (void)dealloc {
	self.document = nil;
    [super dealloc];
}

@end

