/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  PlaygroundViewController.mm
//  aria
//
//  Created by Yohan on 06/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "PlaygroundViewController.h"


@implementation PlaygroundViewController

@synthesize playButton;
@synthesize pauseButton;
@synthesize stopButton;
@synthesize pad1Button;
@synthesize pad2Button;
@synthesize pad3Button;
@synthesize pad4Button;
@synthesize slider;
@synthesize trackpadButton;
@synthesize timeLabel;
@synthesize lastEventLabel;


#pragma mark -
#pragma mark View management

// Do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	// get audio interfaces
	scheduler = [AMSoundManager getScheduler];
	eventCenter = [AMSoundManager getNotificationCenter];
	
	// set actions for toolbar items
	[playButton setTarget:self];
	[playButton setAction:@selector(playButtonPressed)];
	[pauseButton setTarget:self];
	[pauseButton setAction:@selector(pauseButtonPressed)];
	[stopButton setTarget:self];
	[stopButton setAction:@selector(stopButtonPressed)];
	
	// set up time display refresh
	refreshTimer = [NSTimer scheduledTimerWithTimeInterval:0.01 target:self selector:@selector(timeDisplayRefresh:) userInfo:nil repeats:YES];
	
    [super viewDidLoad];
}

- (BOOL) canBecomeFirstResponder { 
    return YES; 
} 

- (void)viewDidAppear:(BOOL)animated { 
    [super viewDidAppear:animated]; 
    [self becomeFirstResponder];		// necessary for shake detection
} 

- (void) viewWillDisappear:(BOOL)animated
{
    [self resignFirstResponder];
    [super viewWillDisappear:animated];
}

- (void)viewWillAppear:(BOOL)animated {
	[slider setHidden:UIInterfaceOrientationIsLandscape(self.interfaceOrientation)];
	[super viewWillAppear:animated];
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
	[slider setHidden:UIInterfaceOrientationIsLandscape(toInterfaceOrientation)];
	[super willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)dealloc {
	// release used vars
	self.playButton = nil;
	self.pauseButton = nil;
	self.stopButton = nil;
	self.pad1Button = nil;
	self.pad2Button = nil;
	self.pad3Button = nil;
	self.pad4Button = nil;
	self.slider = nil;
	self.trackpadButton = nil;
	self.timeLabel = nil;
	self.lastEventLabel = nil;	
	[refreshTimer invalidate];
	
    [super dealloc];
}

#pragma mark -
#pragma mark Internal functions

- (void)sendEvent:(NSString *)_eventName value:(id)_value
{
	[eventCenter postNotificationName:_eventName object:_value];
	
	if (_value != nil) {
		if ([_value isKindOfClass:[NSValue class]]) {
			lastEventLabel.text = [NSString stringWithFormat:@"%@ @ %d", _eventName, [(NSNumber *)_value integerValue]]; 
		} else if ([_value isKindOfClass:[NSArray class]]) {
			lastEventLabel.text = [NSString stringWithFormat:@"%@ @ (%d, %d)", _eventName, [(NSNumber *)[(NSArray *)_value objectAtIndex:0] integerValue], [(NSNumber *)[(NSArray *)_value objectAtIndex:1] integerValue]]; 
		} else {
			lastEventLabel.text = _eventName;
		}
	} else {
		lastEventLabel.text = _eventName;
	}
}

- (void)timeDisplayRefresh:(NSTimer *)timer
{
	int time = scheduler.currentTime * 100.0; 
	timeLabel.text = [NSString stringWithFormat:@"Time : %02d:%02d:%02d", ((time / 100 / 60) % 60), ((time / 100) % 60), (time % 100)];
}

#pragma mark -
#pragma mark UI callbacks

- (void)playButtonPressed
{
	[scheduler start];
	[self sendEvent:@"scheduler.started" value:nil];
}

- (void)pauseButtonPressed
{
	if (!scheduler.paused) {
		[scheduler pause];
		[self sendEvent:@"scheduler.paused" value:nil];
	}
}

- (void)stopButtonPressed
{
	[scheduler resetTime];
	[self sendEvent:@"scheduler.stopped" value:nil];
}

- (IBAction)padPressed:(UIButton *)sender
{
	if (sender == pad1Button) {
		[self sendEvent:@"pad1.pressed" value:nil];
	} else if (sender == pad2Button) {
		[self sendEvent:@"pad2.pressed" value:nil];
	} else if (sender == pad3Button) {
		[self sendEvent:@"pad3.pressed" value:nil];
	} else if (sender == pad4Button) {
		[self sendEvent:@"pad4.pressed" value:nil];
	}
}

- (IBAction)padRelessed:(UIButton *)sender
{
	if (sender == pad1Button) {
		[self sendEvent:@"pad1.released" value:nil];
	} else if (sender == pad2Button) {
		[self sendEvent:@"pad2.released" value:nil];
	} else if (sender == pad3Button) {
		[self sendEvent:@"pad3.released" value:nil];
	} else if (sender == pad4Button) {
		[self sendEvent:@"pad4.released" value:nil];
	}
}

- (IBAction)sliderValueChanged:(UISlider *)sender
{
	[self sendEvent:@"slider.changed" value:[NSNumber numberWithInt:(int)sender.value]];
}

- (IBAction)trackpadDown:(TrackpadButton *)sender
{
	int x = sender.touchedPosition.x - sender.bounds.size.width / 2;
	int y = -(sender.touchedPosition.y - sender.bounds.size.height / 2);
	NSArray *val = [NSArray arrayWithObjects:[NSNumber numberWithInt:x], [NSNumber numberWithInt:y], nil];
	
	[self sendEvent:@"trackpad.changed" value:val];
}

- (void)motionBegan:(UIEventSubtype)motion withEvent:(UIEvent *)event 
{ 
    [self sendEvent:@"accelerometer.shaked" value:nil];
    [super motionEnded:motion withEvent:event]; 
} 


//#pragma mark -
//#pragma mark Accelerometer delegate
//
//- (void)accelerometer:(UIAccelerometer *)accelerometer didAccelerate:(UIAcceleration *)acceleration {
//
//	static float px, py, pz; 
//
//	float dot = (px * acceleration.x) + (py * acceleration.y) + (pz * acceleration.z);
//	float a = ABS(sqrt(px * px + py * py + pz * pz));
//	float b = ABS(sqrt(acceleration.x * acceleration.x + acceleration.y * acceleration.y + acceleration.z * acceleration.z));
//	dot /= (a * b);
//	
//	px = acceleration.x;
//	py = acceleration.y;
//	pz = acceleration.z;
//	
//	NSLog(@"shake dot: %.2f", dot);
//}

@end
