/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  Document.m
//  aria
//
//  Created by Yohan on 08/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "Document.h"


@implementation Document

@synthesize online;
@synthesize name;
@synthesize url;
@synthesize description;


- (BOOL)isInCache 
{
	BOOL cached = !online;
	
	if (!cached) {
		NSString *path = (NSString *)[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
		NSString *filename = [url lastPathComponent];
		
		if (filename && name && ![filename isEqualToString:@""]) {
			// check if it's cached or not
			path = [path stringByAppendingPathComponent:name];
			path = [path stringByAppendingPathComponent:filename];
			cached = [[NSFileManager defaultManager] fileExistsAtPath:path];
		}
	}
	
	return cached;
}

- (NSString *)load:(NSError **)_error
{
	NSMutableString *infos = nil;
	NSString *path = nil;
	
	// check if it's an online document or a local one
	if (online) {
		if ([self isInCache]) {
			// online document, use cached path
			path = (NSString *)[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
			path = [path stringByAppendingPathComponent:name];
			path = [path stringByAppendingPathComponent:[url lastPathComponent]];
		} else {
			return nil;
		}
		
	} else {
		// local document
		path = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:url];
	}
	
	// reset scheduler
	AMScheduler *scheduler = [AMSoundManager getScheduler];
	[scheduler pause];
	[scheduler resetTime];
	
	// load document
	[AMSoundManager loadA2ML:path error:_error];
	
	// write HTML informations
	NSString *contents = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
	
	// escape HTML special chars
	contents = [contents stringByReplacingOccurrencesOfString:@"&" withString:@"&amp;"];
    contents = [contents stringByReplacingOccurrencesOfString:@"\"" withString:@"&quot;"];
    contents = [contents stringByReplacingOccurrencesOfString:@"'" withString:@"&#x27;"];
    contents = [contents stringByReplacingOccurrencesOfString:@">" withString:@"&gt;"];
    contents = [contents stringByReplacingOccurrencesOfString:@"<" withString:@"&lt;"];
	
	// build HTML infos
	infos = [NSMutableString stringWithFormat:@"<html><head><meta name=\"viewport\" content=\"width=device-width; initial-scale=1.0; maximum-scale=2.0; user-scalable=1\"><script type=\"text/javascript\" src=\"shCore.js\"></script><script type=\"text/javascript\" src=\"shBrushXml.js\"></script><link type=\"text/css\" rel=\"stylesheet\" href=\"shCoreDefault.css\"/><script type=\"text/javascript\">SyntaxHighlighter.all();</script></head><body style='font-family:arial'><h1>%@</h1><h2 style='color:#324f85'>Description</h2>%@<h2 style='color:#324f85'>Contents</h2><pre class=\"brush:xml;gutter:false;toolbar:false;\">%@</pre></body></html>", name, [description stringByReplacingOccurrencesOfString:@"\n" withString:@"<br>"], contents];
	
	return infos;
}

- (NSArray *)getSoundsURLs
{	
	NSString *path = nil;
	
	if ([self isInCache]) {
		// online document, use cached path
		path = (NSString *)[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
		path = [path stringByAppendingPathComponent:name];
		path = [path stringByAppendingPathComponent:[url lastPathComponent]];
	}
	
	// can't get the url of sounds if the A2ML file is not cached, and work only for online documents
	if (!path || !online)
		return nil;	
	
	// Gets the base component of the URL
	sndUrls = [NSMutableArray array];
	baseURL = [[url substringToIndex:([url length] - [[url lastPathComponent] length])] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
	if (![baseURL hasSuffix:@"/"] && ![baseURL isEqualToString:@""])
		baseURL = [baseURL stringByAppendingString:@"/"];
	
	AM_LOG(@"Sound base URL: '%@'", baseURL);
	
    // Convert the path to a proper NSURL
    NSURL *xmlURL = [NSURL fileURLWithPath:path];	
    NSXMLParser *parser = [[NSXMLParser alloc] initWithContentsOfURL:xmlURL];
	
    // Set self as the delegate of the parser so that it will receive the parser delegate methods callbacks
    [parser setDelegate:self];
	
    // Configure the parser
    [parser setShouldProcessNamespaces:NO];
    [parser setShouldReportNamespacePrefixes:NO];
    [parser setShouldResolveExternalEntities:NO];
    
	// Parse the document
	AM_LOG(@"Parsing cached path %@", path);
	
    [parser parse];
    [parser release];

	AM_LOG(@"Parsing done, found %d sounds URL.", [sndUrls count]);

	return ([sndUrls count] > 0 ? sndUrls : nil);
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
	if (qName)
        elementName = qName;
	
	if ([elementName isEqualToString:@"sound"]) {
		
		// retrieve sound url
		NSString *sndUrl = [attributeDict valueForKey:@"src"];
		if (sndUrl && [sndUrl length] > 0) {
			sndUrl = [baseURL stringByAppendingString:sndUrl];
			
			// add only unique URLs
			if (![sndUrls containsObject:sndUrl]) 
				[sndUrls addObject:sndUrl];
			
			AM_LOG(@"Found Sound URL: '%@'", sndUrl);
		}
    }
}

@end
