/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  DownloadManager.h
//  aria
//
//  Created by Yohan on 02/08/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import <Foundation/Foundation.h>


// Constants
#define DM_CONNECTION_TIMEOUT	15		// connection timeout


/**
 Manages single and multiple download queues with notification of progress.
 */
@interface DownloadManager : NSObject {
	// Attributes
	id delegate;
	int finishCount;
	NSMutableArray *urls;
	NSMutableArray *receivedDatas;
	
	// Private vars
	NSMutableArray *filePaths;
	NSMutableDictionary *connections;
}

/** The list of URLs to download. */
@property (nonatomic, retain) NSArray *urls;

/** The received data array (if not using a destination path). */
@property (nonatomic, retain) NSArray *receivedDatas;

/** Number of finished downloads. */
@property int finishCount;

/** DownloadManager delegate. */
@property (retain) id delegate;


/**
 Checks if a URL is valid and its protocol supported.
 @param url The URL to check.
 @return YES if the URL is valid, or NO is the URL is invalid or its protocol is unsupported.
 */
+ (BOOL)isValidUrl:(NSString *)_url;

/**
 Downloads the file at the specified URL (synchronously).
 @param url The URL of the file to download.
 @return The data corresponding to the given file
 */
+ (NSData *)downloadFileAtUrl:(NSString *)_url;

/**
 Downloads the files at the given URL list.
 The downloaded data is automatically store in the receivedDatas array in the 
 form of NSData objects. 
 @param urls An array containing the URL of the files to download. 
 */
- (void)downloadFilesAtUrls:(NSArray *)_urls;

/**
 Downloads the files at the given URL list.
 The files will be written at the given path using the filenames contained in
 the URLs.
 @param urls An array containing the URL of the files to download.
 @param path The destination path where you want to save the downloaded files.
 */
- (void)downloadFilesAtUrls:(NSArray *)_urls toPath:(NSString *)_path;

/**
 Gets the downloaded data for the given index.
 The index corresponds to the URL list given to initiate the tranfer.
 @param index The index of the downloaded data to get.
 @return The NSData object corresponding to the index.
 */
- (NSData *)dataAtIndex:(int)_index;

@end


/** DownloadManager delegate methods. */
@protocol DownloadManagerDelegate

/**
 Called when all the downloads have finished.
 */
- (void)didFinishAllDownload;

@optional

/**
 Called when the download corresponding to the URL at the given index has finished.
 @param index The index of download which has finished.
 */
- (void)didFinishDownload:(int)_index;

/**
 Called when the download corresponding to the URL at the given index has failed.
 @param index The index of download which has failed.
 @param error The cause of the failure.
 */
- (void)didFailDownload:(int)_index withError:(NSError *)_error;

@end
