/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  DownloadManager.m
//  aria
//
//  Created by Yohan on 02/08/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "DownloadManager.h"


@implementation DownloadManager

@synthesize urls;
@synthesize receivedDatas;
@synthesize finishCount;
@synthesize delegate;

#pragma mark -
#pragma mark Life cycle

- (id)init {
    if ((self = [super init])) {
		self.urls = nil;
		self.receivedDatas = [NSMutableArray array];
		self.finishCount = 0;
		self.delegate = nil;
		filePaths = nil;
		connections = [[NSMutableDictionary alloc] init];
    }
    return self;
}

- (void)dealloc {
	[filePaths release];
	[connections release];
	self.urls = nil;
	self.receivedDatas = nil;
	self.delegate = nil;
	
    [super dealloc];
}

#pragma mark -
#pragma mark Dowload methods

+ (BOOL)isValidUrl:(NSString *)_url
{
	return [NSURLConnection canHandleRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:_url]]];
}

+ (NSData *)downloadFileAtUrl:(NSString *)_url
{
	NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:_url] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:DM_CONNECTION_TIMEOUT];		
	return [NSURLConnection sendSynchronousRequest:request returningResponse:NULL error:NULL];
}

- (void)downloadFilesAtUrls:(NSArray *)_urls
{
	[self downloadFilesAtUrls:_urls toPath:nil];
}

- (void)downloadFilesAtUrls:(NSArray *)_urls toPath:(NSString *)_path
{
	// init vars
	[receivedDatas removeAllObjects];
	[connections removeAllObjects];
	[urls release];
	urls = [_urls copy];
	[filePaths release];
	filePaths = (_path ? [[NSMutableArray alloc] init]: nil);
	self.finishCount = 0;
	
	int i = 0;
	for (NSString *url in _urls) {
		// init file path or data
		if (filePaths) {
			// delete old file prior downloading
			NSString *path = [_path stringByAppendingPathComponent:[url lastPathComponent]];
			
			if ([[NSFileManager defaultManager] fileExistsAtPath:path])
				 [[NSFileManager defaultManager] removeItemAtPath:path error:NULL];
			
			[filePaths addObject:path];
		} else {
			[receivedDatas addObject: [NSMutableData dataWithLength:0]];
		}
		
		// init download
		NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:url] cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:DM_CONNECTION_TIMEOUT];
		NSURLConnection *connection = [NSURLConnection connectionWithRequest:request delegate:self];
		[connections setObject:[NSNumber numberWithInt:i++] forKey:[NSValue valueWithNonretainedObject:connection]];
	}
}

- (NSData *)dataAtIndex:(int)_index 
{
	return [receivedDatas objectAtIndex:_index];
}

#pragma mark -
#pragma mark NSURLConnection Delegates

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	int i = [[connections objectForKey:[NSValue valueWithNonretainedObject:connection]] intValue];
	
	if (filePaths) {
		NSString *path = [filePaths objectAtIndex:i];
		
		// save to file
		if (![[NSFileManager defaultManager] fileExistsAtPath:path]) {
			[[NSFileManager defaultManager] createFileAtPath:path contents:data attributes:nil];
		} else {
			NSFileHandle *fileHandle = [NSFileHandle fileHandleForUpdatingAtPath:path];
			[fileHandle seekToEndOfFile];
			[fileHandle writeData:data];
			[fileHandle closeFile];
		}
	} else {
		// save to NSData object
		[[receivedDatas objectAtIndex:i] appendData:data];		
	}
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	self.finishCount++;
	
	if ([delegate respondsToSelector:@selector(didFinishDownload:)]) {
		int i = [[connections objectForKey:[NSValue valueWithNonretainedObject:connection]] intValue];
		[delegate didFinishDownload:i];
	}
	
	if(finishCount >= [urls count])
		if ([delegate respondsToSelector:@selector(didFinishAllDownload)])
			[delegate didFinishAllDownload];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	if ([delegate respondsToSelector:@selector(didFailDownloadWithError:)]) {
		int i = [[connections objectForKey:[NSValue valueWithNonretainedObject:connection]] intValue];
        [delegate didFailDownload:i withError:error];		
	}
}

@end
